/*
 * callbacks.c
 * This file is part of gconta
 *
 * Copyright (C) 2004 Bruno Orcha Garcia
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, 
 * Boston, MA 02111-1307, USA. 
 */

#include "callbacks.h"




void
gconta_add_tab		(gpointer		* window,
					 const gchar	* nombre,
					 GtkWidget		* child,
					 GtkWidget		* bar,
					 gpointer		* datos)
{
	GtkWidget * label;
	gint pos, t;
	GtkNotebook *nb = g_object_get_data(G_OBJECT(window), "notebook");
	
	if (nb == NULL) g_error("no hay notebook");
	t = gtk_notebook_get_n_pages(nb);
	if (t == 1 && GTK_IS_IMAGE(gtk_notebook_get_nth_page(nb, 0))) {
		gtk_notebook_remove_page(nb, 0);
		gtk_notebook_set_show_tabs(nb, TRUE);
	}
	
	label = gtk_label_new(nombre);
	gtk_label_set_use_markup(GTK_LABEL(label), TRUE);
	gtk_widget_show(label);
	gtk_widget_show(child);
	
	g_object_set_data(G_OBJECT(child), "barra", (gpointer) bar);
	g_object_set_data(G_OBJECT(child), "tipo", datos);
		
	pos = gtk_notebook_append_page(nb, child, label);
	gtk_notebook_set_current_page(nb, pos);
}
					 


void
boton_pgc			(GtkButton		* boton,
					 gpointer		* window)
{
	GtkWidget * child;
	GtkWidget * barra;
	
	child = pgc_child (window);
	barra = pgc_barra (window);
	barra = gtk_widget_ref(barra);

	gconta_add_tab(window, "PGC", child, (gpointer) barra, GINT_TO_POINTER(GCONTA_PGC));	
}

void
boton_diario		(GtkButton		* boton,
					 gpointer		* window)
{
	GtkWidget * child;
	GtkWidget * barra;
	
	child = diario_child(window);
	barra = diario_barra(window);
	barra = gtk_widget_ref(barra);
	
	gconta_add_tab(window, "Diario", child, (gpointer) barra, GINT_TO_POINTER(GCONTA_DIARIO));
}

void
boton_mayor			(GtkButton		* boton,
					 gpointer		* window)
{
	GtkWidget * child;
	GtkWidget * barra;
	
	child = mayor_child(window);
	barra = mayor_barra(window);
	barra = gtk_widget_ref(barra);

	gconta_add_tab(window, "Mayor", child, (gpointer) barra, GINT_TO_POINTER(GCONTA_MAYOR));
}

void
boton_consultas	(GtkButton	* boton,
				gpointer	* window)
{
	GtkWidget * child;
	GtkWidget * barra;
	
	child = consultas_child(window);
	barra = consultas_barra(window);
	barra = gtk_widget_ref(barra);
	
	gconta_add_tab(window, "Consultas", child, (gpointer) barra, GINT_TO_POINTER(GCONTA_CONSULTAS));	
}


void
boton_listados	(GtkButton	* boton,
				gpointer	* window)
{
	GtkWidget * child;
	GtkWidget * barra;
	
	child = listados_child(window);
	barra = listados_barra(window);
	barra = gtk_widget_ref(barra);
	
	gconta_add_tab(window, "Listados", child, (gpointer) barra, GINT_TO_POINTER(GCONTA_LISTADOS));	
}

void
boton_informes (GtkButton	* boton,
				gpointer	* window)
{
	GtkWidget * child;
	GtkWidget * barra;
	
	child = informes_child(window);
	barra = informes_barra(window);
	barra = gtk_widget_ref(barra);
	
	gconta_add_tab(window, "Informes", child, (gpointer) barra, GINT_TO_POINTER(GCONTA_INFORMES));	
}


void
boton_notas	(GtkButton	* boton,
			gpointer	* window)
{
	GtkWidget * child;
	GtkWidget * barra;
	
	child = notas_child(window);
	barra = notas_barra(window);
	barra = gtk_widget_ref(barra);
	
	gconta_add_tab(window, "Notas", child, (gpointer) barra, GINT_TO_POINTER(GCONTA_NOTAS));	
}


void
solapa_cambia	(GtkNotebook *notebook,
				GtkNotebookPage *p,
				guint arg1,
				gpointer window)
{
	GtkWidget *vbox;
	GtkWidget *child;
	GtkWidget *bar;
	GList *l;
	gint tam;
	

	child = gtk_notebook_get_nth_page(notebook, arg1);
	
	vbox = g_object_get_data(G_OBJECT(window), "vbox2");
	gtk_widget_hide(vbox);
	
	l = gtk_container_get_children(GTK_CONTAINER(vbox));
	tam = g_list_length(l);
	if (tam == 1) {
		bar = (GtkWidget *) l->data;
		gtk_container_remove(GTK_CONTAINER(vbox), bar);
	} else if (tam > 1) g_error("En la barra hay mas de un elemento");
	
	bar = g_object_get_data(G_OBJECT(child), "barra");
	if (bar != NULL) {
		gtk_container_add(GTK_CONTAINER(vbox), bar);
		gtk_widget_show(GTK_WIDGET(vbox));	
	}
}

gboolean
gconta_tecla_pulsada	(GtkWidget *window,
						GdkEventKey *event,
						gpointer user_data)
{
	GtkWidget *child;
	GtkTreeView *treeview;
	gint n;
	
	child = gconta_get_child(window);
	n = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(child), "tipo"));

	switch (n) {
		case GCONTA_DIARIO:
			treeview = g_object_get_data(G_OBJECT(child), "treeview");
			if (gtk_widget_is_focus(GTK_WIDGET(treeview)))
				switch (event->keyval) {
				case GDK_KP_Add:
				case GDK_plus:
					gconta_busca_cuenta(GTK_WINDOW(window));
					return TRUE;
					break;
				case GDK_Insert:
					diario_barra_add(NULL, window);
					return TRUE;
					break;
				case GDK_Delete:
					diario_barra_remove(NULL, window);
					return TRUE;
					break;
				}
			if (event->keyval == GDK_F2) {
				diario_barra_guardar(NULL, window);
				return TRUE;
			}
			break;
		case GCONTA_MAYOR:
			switch (event->keyval) {
				case GDK_Return:
				case GDK_KP_Enter:
					mayor_ver(window);
					break;
				case GDK_KP_Add:
				case GDK_plus:
					gconta_busca_cuenta(GTK_WINDOW(window));
					return TRUE;
				break;
			}
			break;
	}
	
	return FALSE;
}
