/*
 * diario_predefinidos.c
 * This file is part of gconta
 *
 * Copyright (C) 2004 Bruno Orcha Garcia
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, 
 * Boston, MA 02111-1307, USA. 
 */

#include "diario.h"
#include <glib.h>
#include <stdio.h>
//#include <string.h>
#include <gdk/gdkkeysyms.h>


// Variables globales para el analis léxico y sintáctico
enum { ID = 1, FIN };
gchar *gconta_texto;
int gconta_pos;
int gconta_preanalisis;
GList *gconta_ts;
GList *gconta_tp;
gboolean gconta_anasin_error;

/***********************************************************/

void
diario_predefinidos_free_tpartidas(GList *l_tp)
{
	GList *node;
	tpartidas *tp;
	
	for (node = g_list_first(l_tp); node != NULL; node = g_list_next(node)) {
		tp = (tpartidas *) node->data;
		g_free(tp->cuenta);
		g_free(tp->cuantia);
		g_free(tp);
	}
	g_list_free(l_tp);
}

void
diario_predefinidos_free_tsimb(GList *l_ts)
{
	GList *node;
	tsimb *ts;
	
	for (node = g_list_first(l_ts); node != NULL; node = g_list_next(node)) {
		ts = (tsimb *) node->data;
		g_free(ts->var);
		g_free(ts->question);
		g_free(ts);
	}
	g_list_free(l_ts);
}

void
diario_barra_combo_changed	(GtkComboBox *combobox,
							gpointer window)
{
	GtkWidget *child;
	GtkWidget *bar;
	GtkWidget *dialog;
	GtkComboBox *combo;
	GtkListStore *store;
	GtkTreeIter iter;
	gchar *buffer;
		
	child = gconta_get_child(window);
	bar = g_object_get_data(G_OBJECT(child), "barra");
	combo = g_object_get_data(G_OBJECT(bar), "combobox1");
	store = (GtkListStore*) gtk_combo_box_get_model(combo);
	if (gtk_combo_box_get_active_iter(combo, &iter)) {
		gtk_tree_model_get(GTK_TREE_MODEL(store), &iter, 0, &buffer, -1);
		if (strlen(buffer) == 0) return;
		gconta_ts = NULL;
		gconta_tp = NULL;
		xml_diario_predefinidos_llenar(&gconta_ts, &gconta_tp, buffer);
		if (diario_predefinidos_anasin() != -1) {
			diario_predefinidos_rellenar_variables();
			diario_predefinidos_resolver_cuantias();
			diario_predefinidos_add_partidas(window);
		} else {
			dialog = create_dialog ("Ocurrió un error al procesar los predefinidos", TRUE);
			gtk_dialog_run(GTK_DIALOG(dialog));
			gtk_widget_destroy(dialog);
		}
		diario_predefinidos_free_tpartidas(gconta_tp);
		diario_predefinidos_free_tsimb(gconta_ts);
		g_free(buffer);
	}
}

gboolean
diario_pregunta_key (GtkDialog *dialog,
					GdkEventKey *event,
					gpointer data)
{
	switch (event->keyval) {
	case GDK_Return:
	case GDK_KP_Enter:
		
			gtk_dialog_response(dialog, GTK_RESPONSE_OK);
			return TRUE;
		break;
	}
	return FALSE;
}

GtkWidget*
diario_pregunta (gchar *texto)
{
  GtkWidget *dialog1;
  GtkWidget *dialog_vbox1;
  GtkWidget *vbox1;
  GtkWidget *label1;
  GtkWidget *entry1;
  GtkWidget *dialog_action_area1;
  GtkWidget *cancelbutton1;
  GtkWidget *okbutton1;

  dialog1 = gtk_dialog_new ();
  gtk_container_set_border_width (GTK_CONTAINER (dialog1), 6);
  gtk_window_set_resizable (GTK_WINDOW (dialog1), FALSE);
  gtk_window_set_type_hint (GTK_WINDOW (dialog1), GDK_WINDOW_TYPE_HINT_DIALOG);

  dialog_vbox1 = GTK_DIALOG (dialog1)->vbox;
  gtk_widget_show (dialog_vbox1);

  vbox1 = gtk_vbox_new (FALSE, 12);
  gtk_widget_show (vbox1);
  gtk_box_pack_start (GTK_BOX (dialog_vbox1), vbox1, TRUE, TRUE, 0);

  label1 = gtk_label_new (texto);
  gtk_widget_show (label1);
  gtk_box_pack_start (GTK_BOX (vbox1), label1, FALSE, FALSE, 0);
  gtk_label_set_use_markup (GTK_LABEL (label1), TRUE);

  entry1 = gtk_entry_new ();
  gtk_widget_show (entry1);
  gtk_box_pack_start (GTK_BOX (vbox1), entry1, FALSE, FALSE, 0);

  dialog_action_area1 = GTK_DIALOG (dialog1)->action_area;
  gtk_widget_show (dialog_action_area1);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_area1), GTK_BUTTONBOX_END);

  cancelbutton1 = gtk_button_new_from_stock ("gtk-cancel");
  gtk_widget_show (cancelbutton1);
  gtk_dialog_add_action_widget (GTK_DIALOG (dialog1), cancelbutton1, GTK_RESPONSE_CANCEL);
  GTK_WIDGET_SET_FLAGS (cancelbutton1, GTK_CAN_DEFAULT);

  okbutton1 = gtk_button_new_from_stock ("gtk-ok");
  gtk_widget_show (okbutton1);
  gtk_dialog_add_action_widget (GTK_DIALOG (dialog1), okbutton1, GTK_RESPONSE_OK);
  GTK_WIDGET_SET_FLAGS (okbutton1, GTK_CAN_DEFAULT);
  
  g_object_set_data(G_OBJECT(dialog1), "entry1", entry1);
  
  g_signal_connect(dialog1, "key-press-event", G_CALLBACK(diario_pregunta_key), NULL);
  
  return dialog1;
}



void expresion ();

int
gconta_tabla_add(const char *buffer)
{
	GList *node;
	tsimb *t;
	int mpos = 100;
	
	for(node = g_list_first(gconta_ts); node != NULL; node = g_list_next(node))
		if (strcmp(((tsimb *) node->data)->var, buffer) == 0) {
			return mpos;
		} else mpos++;

	t = g_new(tsimb, 1);
	t->var = g_strdup(buffer);
	t->question = NULL;
	t->val = 2;
	gconta_ts = g_list_append(gconta_ts, t);
		
	return mpos;
}



gdouble
gconta_tabla_get(int code)
{
	tsimb *t;
	
	if (gconta_ts == NULL || code < 100) return 0;
	t = g_list_nth_data(gconta_ts, code - 100);

	if (t == NULL) return 0;
	return t->val;
}


// Analisis lexico
int
gconta_analex(int *code)
{
	gchar buffer[10];
	gint buffer_pos = 0;
	
	while(gconta_texto[gconta_pos] == ' ') gconta_pos++;
	if (gconta_texto[gconta_pos] == '\0') return FIN;
		
	if (g_ascii_isalpha(gconta_texto[gconta_pos])) {
		// Variable
		while(g_ascii_isalnum(gconta_texto[gconta_pos])) {
			buffer[buffer_pos] = gconta_texto[gconta_pos];
			buffer_pos++;
			gconta_pos++;
		}
		buffer[buffer_pos] = '\0';
		// añadir buffer a la tabla de simbolos
		*code = gconta_tabla_add(buffer);
		return ID;
	} else if (g_ascii_isdigit(gconta_texto[gconta_pos])) {
		// Numero
		while (g_ascii_isdigit(gconta_texto[gconta_pos])) {
			buffer[buffer_pos] = gconta_texto[gconta_pos];
			buffer_pos++;
			gconta_pos++;
		}
		if (gconta_texto[gconta_pos] == '.') {
			buffer[buffer_pos] = gconta_texto[gconta_pos];
			buffer_pos++;
			gconta_pos++;
			while (g_ascii_isdigit(gconta_texto[gconta_pos])) {
				buffer[buffer_pos] = gconta_texto[gconta_pos];
				buffer_pos++;
				gconta_pos++;
			}
		}
		buffer[buffer_pos] = '\0';
		*code = gconta_tabla_add(buffer);
		return ID;
	} else {
		switch (gconta_texto[gconta_pos]) {
		case '+':
		case '-':
		case '/':
		case '*':
		case '(':
		case ')':
			gconta_pos++;
			return gconta_texto[gconta_pos - 1];
			break;

		default:
			return -1;
		}
	}
}

// Funciones para analisis semantico
void reconocer(int t)
{
	int i;
	
	if (gconta_preanalisis == t) {
		gconta_preanalisis = gconta_analex(&i);
	}
	else
		gconta_anasin_error = TRUE;
}

void op ()
{
	if (gconta_anasin_error) return;
	switch(gconta_preanalisis) {
		case '+':
			reconocer('+');
			break;
		case '-':
			reconocer('-');
			break;
		case '/':
			reconocer('/');
			break;
		case '*':
			reconocer('*');
			break;
		default:
			gconta_anasin_error = TRUE;
	}
}

void resto ()
{
	if (gconta_anasin_error) return;
	switch(gconta_preanalisis) {
		case '+':
		case '-':
		case '/':
		case '*':
			op();
			expresion();
			break;
		case FIN:
		case ')':
			break;
		default:
			gconta_anasin_error = TRUE;
	}		
}

void expresion2 ()
{
	if (gconta_anasin_error) return;
	switch(gconta_preanalisis) {
		case ID:
			reconocer(ID);
			break;
		case '(':
			reconocer('(');
			expresion();
			reconocer(')');
			break;
		default:
			gconta_anasin_error = TRUE;
	}
}

void expresion ()
{
	if (gconta_anasin_error) return;
	expresion2();
	resto();
}

void meta()
{
	expresion();
	reconocer(FIN);
}

/**********************************************************************************/
// Transformación de expresion infica a postfija
int
prioridad(char c)
{
	int k = 0;
	
	switch(c) {
		case '+':
		case '-':
			k = 1;
			break;
		case '*':
		case '/':
			k = 2;
			break;
	}
return k;
}

void
postfija(GQueue *f)
{
int val, code;
GQueue *q;
gconta_pos = 0;

q = g_queue_new();

while ((val = gconta_analex(&code)) != FIN) {
	switch(val)	{
	case ID:
		g_queue_push_head(f, GINT_TO_POINTER(code));
		break;
	case '(':
		g_queue_push_head(q, GINT_TO_POINTER('('));
		break;
	case ')':
		while (GPOINTER_TO_INT(g_queue_peek_head(q)) != '(') 
			g_queue_push_head(f, g_queue_pop_head(q));
		g_queue_pop_head(q);
		break;
	case '+':
	case '-':
	case '*':
	case '/':
		while (!g_queue_is_empty(q) && GPOINTER_TO_INT(g_queue_peek_head(q)) != '(' &&
					prioridad(GPOINTER_TO_INT(g_queue_peek_head(q))) >= prioridad(val)) 
			g_queue_push_head(f, g_queue_pop_head(q));
		g_queue_push_head(q, GINT_TO_POINTER(val));
		break;
	}
}
// Agregar a EP los operadores restantes en la Pila
while(!g_queue_is_empty(q)) 
	g_queue_push_head(f, g_queue_pop_head(q));

}
/*********************************************************************************/

gdouble
resul(GQueue *f)
{
	GQueue *g;
	gdouble total;
	gdouble *a, *b;
	int t;
	
	g = g_queue_new();
	total = 0;
	while(!g_queue_is_empty(f)) {
		t = GPOINTER_TO_INT(g_queue_pop_tail(f));
		if (t != '+' && t != '-' && t != '*' && t != '/') {
			a = g_new(gdouble, 1);
			*a = gconta_tabla_get(t); // buscar en la tabla de simbolos
			g_queue_push_head(g, a);
		} else {
			a = (gdouble *) g_queue_pop_head(g);
			b = (gdouble *) g_queue_pop_head(g);
			switch(t) {
				case '+': *a = *b + *a; break;
				case '-': *a = *b - *a;	break;
				case '*': *a = *b * *a;	break;
				case '/':
					if (*a == 0) return 0;
					*a = *b / *a;
					break;
			}
			g_free(b);
			g_queue_push_head(g, a);
		}
	}
	
	a = g_queue_pop_head(g);
	total = *a;
	g_free(a);
	return total;
}

void
diario_predefinidos_rellenar_variables()
{
	GtkWidget *dialog;
	GList *node;
	gboolean ok;
	tsimb *t;
	gchar *txt;

	for(node = g_list_first(gconta_ts); node != NULL; node = g_list_next(node)) {
		t = (tsimb *) node->data;
		if (g_ascii_strtod(t->var, NULL) != 0) t->val = g_ascii_strtod(t->var, NULL);
		else {
			dialog = diario_pregunta(t->question);
			ok = FALSE;
			do {
				if (gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_OK) {
					txt = (gchar *) gtk_entry_get_text(GTK_ENTRY(
							g_object_get_data(G_OBJECT(dialog), "entry1")));
					if (g_ascii_strtod(txt, NULL) > 0) {
						t->val = g_ascii_strtod(txt, NULL);
						ok = TRUE;
					}
				} else ok = TRUE;
			} while (!ok);
			gtk_widget_destroy(dialog);
		}
	}
}

void
diario_predefinidos_resolver_cuantias ()
{
	GQueue *f;
	GList *node;
	tpartidas *tp;
	gchar buffer[MAX_BUFFER];
	int i;

	for(node = g_list_first(gconta_tp); node != NULL; node = g_list_next(node)) {
		tp = (tpartidas *) node->data;
		if (g_ascii_strtod(tp->cuenta, NULL) == 0) {
			i = gconta_tabla_get(gconta_tabla_add(tp->cuenta));
			g_free(tp->cuenta);
			if (i == 0) tp->cuenta = "";
			else {
				g_snprintf(buffer, MAX_BUFFER, "%d", i);
				tp->cuenta = g_strdup(buffer);
			}
		}
		
		if (strlen(tp->cuantia) != 0) {
			gconta_texto = tp->cuantia;
			gconta_pos = 0;
			f = g_queue_new();
			postfija(f);
			g_snprintf(buffer, MAX_BUFFER, "%f", resul(f));
			g_free(tp->cuantia);
			tp->cuantia = g_strdup(buffer);
		}
	}
}

int
diario_predefinidos_anasin ()
{
	GList *node;
	tpartidas *tp;
	int i;

	for(node = g_list_first(gconta_tp); node != NULL; node = g_list_next(node)) {
		tp = (tpartidas *) node->data;
		if (strlen(tp->cuantia) != 0) {
			gconta_texto = tp->cuantia;
			gconta_pos = 0;
			gconta_anasin_error = FALSE;
			gconta_preanalisis = gconta_analex(&i);
			meta();
			if (gconta_anasin_error) return -1;
		}
	}
	return 0;
}

void
diario_predefinidos_add_partidas	(gpointer window)
{
	GList *node, *tmp;
	GtkWidget *child;
	GtkListStore *store;
	GtkTreeView *treeview;
	GtkTreeIter iter;
	GtkTreePath *path, *path1;
	GtkTreeViewColumn *col;
	tpartidas *tp;
	gchar *text_path;

	child = gconta_get_child(window);
	treeview = g_object_get_data (G_OBJECT (child), "treeview");
	store = (GtkListStore *) gtk_tree_view_get_model (treeview);

	diario_barra_limpiar (NULL, window);
	
	for(node = g_list_first(gconta_tp); node != NULL; node = g_list_next(node)) {
		tp = (tpartidas *) node->data;
		
		// Añado la linea de la partida
		gtk_list_store_append (store, &iter);
		gtk_list_store_set (store, &iter, 0, "", 1, "- XXX -", 2, "", 3, "", -1);
		path = gtk_tree_model_get_path (GTK_TREE_MODEL (store), &iter);
		text_path = gtk_tree_path_to_string(path);
		
		// Edito la cuenta
		col = gtk_tree_view_get_column (treeview, 0);
		tmp = gtk_tree_view_column_get_cell_renderers (col);
		diario_editar_cuenta(tmp->data, text_path, tp->cuenta, treeview);
		g_list_free(tmp);
		
		// Edito la cuantia
		if (tp->debe) col = gtk_tree_view_get_column (treeview, 2);
		else col = gtk_tree_view_get_column (treeview, 3);
		tmp = gtk_tree_view_column_get_cell_renderers (col);
		diario_editar_cuantia(tmp->data, text_path, tp->cuantia, window);
		g_list_free(tmp);
		
		// Al editar correctamente una cuantia se añade una partida
		// automaticamente, eliminarla si la ultima mesa - XXX -
		// comprobar el cursor con el iter siguiente al ultimo

		gtk_tree_view_get_cursor (treeview, &path1, NULL);
		if (gtk_tree_path_compare(path, path1) != 0)
			diario_barra_remove(NULL, window);
	}
	
	gtk_widget_grab_focus(GTK_WIDGET(treeview));
}

void
diario_predefinidos_editar_llenar(GtkListStore *store)
{
	gchar *text;
	GtkTreeIter iter;
	gint i;
	GList *sl;
	GList *ts, *tp;
	tpredefinido *pred;
	
	sl = g_object_get_data(G_OBJECT(store), "lista");
	
	gtk_tree_model_get_iter_first(GTK_TREE_MODEL(store), &iter);
	for (i = 0; i < gtk_tree_model_iter_n_children(GTK_TREE_MODEL(store), NULL); i++) {
		gtk_tree_model_get(GTK_TREE_MODEL(store), &iter, 0, &text, -1);
		ts = NULL;
		tp = NULL;
		xml_diario_predefinidos_llenar(&ts, &tp, text);
		pred = g_new(tpredefinido, 1);
		pred->nombre = g_strdup(text);
		pred->ts = ts;
		pred->tp = tp;
		sl = g_list_append(sl, pred);
		g_free(text);
		gtk_tree_model_iter_next(GTK_TREE_MODEL(store), &iter);
	}
	
	g_object_set_data(G_OBJECT(store), "lista", sl);
}

void
diario_predefinidos_editar (GtkToolButton *b,
							gpointer parent)
{
	GtkWidget *window1;
	GtkWidget *vbox1;
	GtkWidget *toolbar1;
	gint tmp_toolbar_icon_size;
	GtkWidget *toolbutton1;
	GtkWidget *toolbutton2;
	GtkWidget *separatortoolitem1;
	GtkWidget *toolbutton4;
	GtkWidget *toolbutton5;
	GtkWidget *toolitem1;
	GtkWidget *label5;
	GtkWidget *toolitem2;
	GtkWidget *combobox1;
	GtkWidget *vbox2;
	GtkWidget *label2;
	GtkWidget *scrolledwindow2;
	GtkWidget *treeview2;
	GtkWidget *label1;
	GtkWidget *scrolledwindow1;
	GtkWidget *treeview1;
	GtkWidget *hbuttonbox1;
	GtkWidget *button1;
	GtkWidget *button2;
	GtkCellRenderer *rend;
	GtkTreeViewColumn *col;
	GtkListStore *store;

	window1 = gtk_window_new (GTK_WINDOW_TOPLEVEL);
	gtk_window_set_title (GTK_WINDOW (window1), _("Editor de predefinidos"));
	gtk_window_set_default_size (GTK_WINDOW (window1), 600, 400);
	
	vbox1 = gtk_vbox_new (FALSE, 0);
	gtk_widget_show (vbox1);
	gtk_container_add (GTK_CONTAINER (window1), vbox1);
	
	toolbar1 = gtk_toolbar_new ();
	gtk_widget_show (toolbar1);
	gtk_box_pack_start (GTK_BOX (vbox1), toolbar1, FALSE, FALSE, 0);
	gtk_toolbar_set_style (GTK_TOOLBAR (toolbar1), GTK_TOOLBAR_ICONS);
	tmp_toolbar_icon_size = gtk_toolbar_get_icon_size (GTK_TOOLBAR (toolbar1));
	
	toolbutton1 = (GtkWidget*) gtk_tool_button_new_from_stock ("gtk-add");
	gtk_widget_show (toolbutton1);
	gtk_container_add (GTK_CONTAINER (toolbar1), toolbutton1);
	
	toolbutton2 = (GtkWidget*) gtk_tool_button_new_from_stock ("gtk-remove");
	gtk_widget_show (toolbutton2);
	gtk_container_add (GTK_CONTAINER (toolbar1), toolbutton2);
	
	separatortoolitem1 = (GtkWidget*) gtk_separator_tool_item_new ();
	gtk_widget_show (separatortoolitem1);
	gtk_container_add (GTK_CONTAINER (toolbar1), separatortoolitem1);
	
	toolbutton4 = (GtkWidget*) gtk_tool_button_new_from_stock ("gtk-new");
	gtk_widget_show (toolbutton4);
	gtk_container_add (GTK_CONTAINER (toolbar1), toolbutton4);
	
	toolbutton5 = (GtkWidget*) gtk_tool_button_new_from_stock ("gtk-delete");
	gtk_widget_show (toolbutton5);
	gtk_container_add (GTK_CONTAINER (toolbar1), toolbutton5);
	
	toolitem1 = (GtkWidget*) gtk_tool_item_new ();
	gtk_widget_show (toolitem1);
	gtk_container_add (GTK_CONTAINER (toolbar1), toolitem1);
	
	label5 = gtk_label_new (_("Predefinido a editar"));
	gtk_widget_show (label5);
	gtk_container_add (GTK_CONTAINER (toolitem1), label5);
	gtk_misc_set_padding (GTK_MISC (label5), 6, 0);
	
	toolitem2 = (GtkWidget*) gtk_tool_item_new ();
	gtk_widget_show (toolitem2);
	gtk_container_add (GTK_CONTAINER (toolbar1), toolitem2);
	
	store = gtk_list_store_new(1, G_TYPE_STRING);
	xml_diario_predefinidos(store);
	// En store tengo la propiedad lista
	diario_predefinidos_editar_llenar(store);

	combobox1 = gtk_combo_box_new_with_model(GTK_TREE_MODEL(store));
	g_object_unref (G_OBJECT (store));
	gtk_widget_show (combobox1);
	gtk_container_add(GTK_CONTAINER(toolitem2), combobox1);
	
	rend = gtk_cell_renderer_text_new ();
	gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (combobox1), rend, TRUE);
	gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT (combobox1), rend,
									"text", 0,
									NULL);
									
	vbox2 = gtk_vbox_new (FALSE, 12);
	gtk_widget_show (vbox2);
	gtk_box_pack_start (GTK_BOX (vbox1), vbox2, TRUE, TRUE, 0);
	gtk_container_set_border_width (GTK_CONTAINER (vbox2), 6);
	
	label2 = gtk_label_new (_("<b>Partidas</b>"));
	gtk_widget_show (label2);
	gtk_box_pack_start (GTK_BOX (vbox2), label2, FALSE, FALSE, 0);
	gtk_label_set_use_markup (GTK_LABEL (label2), TRUE);
	gtk_misc_set_alignment (GTK_MISC (label2), 0, 0.5);
	
	scrolledwindow1 = gtk_scrolled_window_new (NULL, NULL);
	gtk_widget_show (scrolledwindow1);
	gtk_box_pack_start (GTK_BOX (vbox2), scrolledwindow1, TRUE, TRUE, 0);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow1), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
	
	store = gtk_list_store_new (4, G_TYPE_STRING, G_TYPE_STRING,
				    G_TYPE_STRING, G_TYPE_STRING);

	treeview1 = gtk_tree_view_new_with_model (GTK_TREE_MODEL (store));
	gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (treeview1), TRUE);
	gtk_tree_view_set_reorderable (GTK_TREE_VIEW (treeview1), TRUE);
	g_object_unref (G_OBJECT (store));
	gtk_container_add (GTK_CONTAINER (scrolledwindow1), treeview1);	
	gtk_widget_show (treeview1);

	rend = gtk_cell_renderer_text_new ();
	g_object_set_data (G_OBJECT (rend), "col", GINT_TO_POINTER (0));
	g_object_set (G_OBJECT (rend), "editable", TRUE, NULL);
	col = gtk_tree_view_column_new_with_attributes ("Cuenta", rend,
							"text", 0, NULL);
	gtk_tree_view_column_set_resizable (col, TRUE);
	gtk_tree_view_append_column (GTK_TREE_VIEW (treeview1), col);
	g_signal_connect (rend, "edited", G_CALLBACK (diario_predefinidos_editar_col),
			treeview1);

	rend = gtk_cell_renderer_text_new ();
	g_object_set_data (G_OBJECT (rend), "col", GINT_TO_POINTER (1));
	g_object_set (G_OBJECT (rend), "editable", TRUE, NULL);
	col = gtk_tree_view_column_new_with_attributes ("Debe", rend,
							"text", 1, NULL);
	gtk_tree_view_column_set_resizable (col, TRUE);
	gtk_tree_view_append_column (GTK_TREE_VIEW (treeview1), col);
	g_signal_connect (rend, "edited", G_CALLBACK (diario_predefinidos_editar_col),
			treeview1);
	
	rend = gtk_cell_renderer_text_new ();
	g_object_set_data (G_OBJECT (rend), "col", GINT_TO_POINTER (2));
	g_object_set (G_OBJECT (rend), "editable", TRUE, NULL);
	col = gtk_tree_view_column_new_with_attributes ("Haber", rend,
							"text", 2, NULL);
	gtk_tree_view_column_set_resizable (col, TRUE);
	gtk_tree_view_append_column (GTK_TREE_VIEW (treeview1), col);
	g_signal_connect (rend, "edited", G_CALLBACK (diario_predefinidos_editar_col),
			treeview1);

	rend = gtk_cell_renderer_text_new ();
	col = gtk_tree_view_column_new_with_attributes ("", rend,
							"text", 3, NULL);
	gtk_tree_view_column_set_resizable (col, TRUE);
	gtk_tree_view_append_column (GTK_TREE_VIEW (treeview1), col);

	label1 = gtk_label_new (_("<b>Variables</b>"));
	gtk_widget_show (label1);
	gtk_box_pack_start (GTK_BOX (vbox2), label1, FALSE, FALSE, 0);
	gtk_label_set_use_markup (GTK_LABEL (label1), TRUE);
	gtk_misc_set_alignment (GTK_MISC (label1), 0, 0.5);
	
	scrolledwindow2 = gtk_scrolled_window_new (NULL, NULL);
	gtk_widget_show (scrolledwindow2);
	gtk_box_pack_start (GTK_BOX (vbox2), scrolledwindow2, FALSE, TRUE, 0);
	gtk_widget_set_size_request (scrolledwindow2, -1, 114);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow2), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
	
	store = gtk_list_store_new (2, G_TYPE_STRING, G_TYPE_STRING);

	treeview2 = gtk_tree_view_new_with_model (GTK_TREE_MODEL (store));
	gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (treeview2), TRUE);
	gtk_tree_view_set_reorderable (GTK_TREE_VIEW (treeview2), TRUE);
	g_object_unref (G_OBJECT (store));
	gtk_container_add (GTK_CONTAINER (scrolledwindow2), treeview2);	
	gtk_widget_show (treeview2);

	rend = gtk_cell_renderer_text_new ();
	col = gtk_tree_view_column_new_with_attributes ("Variable", rend,
							"text", 0, NULL);
	gtk_tree_view_column_set_resizable (col, TRUE);
	gtk_tree_view_append_column (GTK_TREE_VIEW (treeview2), col);

	rend = gtk_cell_renderer_text_new ();
	g_object_set_data (G_OBJECT (rend), "col", GINT_TO_POINTER (1));
	g_object_set (G_OBJECT (rend), "editable", TRUE, NULL);
	col = gtk_tree_view_column_new_with_attributes ("Pregunta", rend,
							"text", 1, NULL);
	gtk_tree_view_column_set_resizable (col, TRUE);
	gtk_tree_view_append_column (GTK_TREE_VIEW (treeview2), col);
	g_signal_connect (rend, "edited", G_CALLBACK (diario_predefinidos_editar_question),
			treeview2);
	
	hbuttonbox1 = gtk_hbutton_box_new ();
	gtk_widget_show (hbuttonbox1);
	gtk_box_pack_start (GTK_BOX (vbox2), hbuttonbox1, FALSE, TRUE, 0);
	gtk_button_box_set_layout (GTK_BUTTON_BOX (hbuttonbox1), GTK_BUTTONBOX_END);
	gtk_box_set_spacing (GTK_BOX (hbuttonbox1), 12);
	
	button1 = gtk_button_new_from_stock ("gtk-cancel");
	gtk_widget_show (button1);
	gtk_container_add (GTK_CONTAINER (hbuttonbox1), button1);
	GTK_WIDGET_SET_FLAGS (button1, GTK_CAN_DEFAULT);
	
	button2 = gtk_button_new_from_stock ("gtk-save");
	gtk_widget_show (button2);
	gtk_container_add (GTK_CONTAINER (hbuttonbox1), button2);
	GTK_WIDGET_SET_FLAGS (button2, GTK_CAN_DEFAULT);
	
	gtk_widget_show(window1);
	
	g_object_set_data(G_OBJECT(window1), "combobox1", combobox1);
	g_object_set_data(G_OBJECT(window1), "treeview1", treeview1);
	g_object_set_data(G_OBJECT(window1), "treeview2", treeview2);
	g_object_set_data(G_OBJECT(window1), "parent", parent);
	g_object_set_data(G_OBJECT(treeview1), "parent", window1);
	g_object_set_data(G_OBJECT(treeview2), "parent", window1);
	
	
	g_signal_connect(combobox1, "changed", G_CALLBACK(diario_predefinidos_editar_combo), window1);
	g_signal_connect(toolbutton1, "clicked", G_CALLBACK(diario_predefinidos_editar_add), window1);
	g_signal_connect(toolbutton2, "clicked", G_CALLBACK(diario_predefinidos_editar_remove), window1);
	g_signal_connect(toolbutton4, "clicked", G_CALLBACK(diario_predefinidos_editar_new), window1);
	g_signal_connect(toolbutton5, "clicked", G_CALLBACK(diario_predefinidos_editar_eliminar), window1);
	g_signal_connect(button1, "clicked", G_CALLBACK(diario_predefinidos_editar_boton_exit), window1);
	g_signal_connect(button2, "clicked", G_CALLBACK(diario_predefinidos_editar_save), window1);
	g_signal_connect(window1, "delete-event", G_CALLBACK(diario_predefinidos_editar_exit), parent);
}

void
diario_predefinidos_editar_add(GtkToolButton *b, gpointer *window)
{
	GtkComboBox *combobox;
	GtkTreeView *treeview;
	GtkListStore *store;
	GtkTreeIter iter;
	gchar *name;
	
	combobox = g_object_get_data(G_OBJECT(window), "combobox1");
	store = (GtkListStore *) gtk_combo_box_get_model(combobox);
	gtk_combo_box_get_active_iter(combobox, &iter);
	if (!gtk_list_store_iter_is_valid(store, &iter)) return;
	gtk_tree_model_get(GTK_TREE_MODEL(store), &iter, 0, &name, -1);
	if (name == NULL || strlen(name) == 0) return;
	treeview = g_object_get_data(G_OBJECT(window), "treeview1");
	store = (GtkListStore *) gtk_tree_view_get_model(treeview);
	gtk_list_store_append(store, &iter);
	gtk_list_store_set(store, &iter, 0, "XXX", 1, "XX", 2, "", -1);

	diario_predefinidos_editar_guardar_cambios (window);
}

void
diario_predefinidos_editar_remove(GtkToolButton *b, gpointer *window)
{
	GtkTreeView *treeview;
	GtkTreeModel *store;
	GtkTreeIter iter;
	GtkTreePath *path;
	
	treeview = g_object_get_data(G_OBJECT(window), "treeview1");
	store = gtk_tree_view_get_model(treeview);
	
	gtk_tree_view_get_cursor(treeview, &path, NULL);
	if (path != NULL && gtk_tree_model_get_iter(store, &iter, path)) {
		gtk_list_store_remove(GTK_LIST_STORE(store), &iter);
		diario_predefinidos_editar_guardar_cambios (window);
		gtk_tree_path_prev (path);
		gtk_tree_view_set_cursor (treeview, path, NULL, FALSE);
		gtk_widget_grab_focus (GTK_WIDGET (treeview));
	}
}

void
diario_predefinidos_editar_combo(GtkComboBox *b, gpointer *window)
{
	GtkTreeView *treeview;
	GtkTreeModel *store, *store1, *store2;
	GList *lista, *node, *node2;
	GtkTreeIter iter;
	gchar *name;
	tpredefinido *pred;
	tpartidas *tp;
	tsimb *ts;
	gint n;
	
	store = gtk_combo_box_get_model(b);
	treeview = g_object_get_data(G_OBJECT(window), "treeview1");
	store1 = gtk_tree_view_get_model(treeview);
	gtk_list_store_clear(GTK_LIST_STORE(store1));
	treeview = g_object_get_data(G_OBJECT(window), "treeview2");
	store2 = gtk_tree_view_get_model(treeview);
	gtk_list_store_clear(GTK_LIST_STORE(store2));
	lista = g_object_get_data(G_OBJECT(store), "lista");
	gtk_combo_box_get_active_iter(b, &iter);
	gtk_tree_model_get(store, &iter, 0, &name, -1);
	
	for(node = g_list_first(lista); node != NULL; node = g_list_next(node)) {
		pred = (tpredefinido *) node->data;
		if (g_ascii_strcasecmp(pred->nombre, name) == 0) {
			for (node2 = g_list_first(pred->tp); node2 != NULL; node2 = g_list_next(node2)) {
				tp = (tpartidas *) node2->data;
				n = (tp->debe) ? 1 : 2;
				gtk_list_store_append(GTK_LIST_STORE(store1), &iter);
				gtk_list_store_set(GTK_LIST_STORE(store1), &iter, 0, tp->cuenta, n, tp->cuantia, -1);
			}
			for (node2 = g_list_first(pred->ts); node2 != NULL; node2 = g_list_next(node2)) {
				ts = (tsimb *) node2->data;
				gtk_list_store_append(GTK_LIST_STORE(store2), &iter);
				gtk_list_store_set(GTK_LIST_STORE(store2), &iter, 0, ts->var, 1, ts->question, -1);
			}

		}
	}
}

void
diario_predefinidos_editar_boton_exit	(GtkButton *b,
										gpointer window)
{
	gboolean ok;
	GtkWidget *parent;
	
	parent = g_object_get_data(G_OBJECT(window), "parent");
	g_signal_emit_by_name(GTK_WINDOW(window), "delete-event", parent, &ok);
}

gboolean
diario_predefinidos_editar_exit	(GtkWidget *w,
								GdkEvent *event,
								gpointer parent)
{
	GtkWidget *child;
	GtkWidget *barra;
	GtkComboBox *combobox;
	GtkTreeModel *store;
	GList *l, *node;
	tpredefinido *pred;
	
	combobox = g_object_get_data(G_OBJECT(w), "combobox1");
	store = gtk_combo_box_get_model(combobox);
	l = g_object_get_data(G_OBJECT(store), "lista");
	
	for(node = g_list_first(l); node != NULL; node = g_list_next(node)) {
		pred = (tpredefinido *) node->data;
		g_free(pred->nombre);
		diario_predefinidos_free_tpartidas(pred->tp);
		diario_predefinidos_free_tsimb(pred->ts);
		g_free(pred);
	}
	g_list_free(l);
	
	gtk_widget_destroy(GTK_WIDGET(w));
	
	child = gconta_get_child(parent);
	barra = g_object_get_data(G_OBJECT(child), "barra");
	combobox = g_object_get_data(G_OBJECT(barra), "combobox1");
	store = gtk_combo_box_get_model(combobox);
	gtk_list_store_clear(GTK_LIST_STORE(store));
	xml_diario_predefinidos(GTK_LIST_STORE(store));
	
	return TRUE;
}

void
diario_predefinidos_editar_col	(GtkCellRendererText * cellrenderertext,
		      					gchar * path,
								gchar * text,
								gpointer treeview)
{
	GtkWidget *window;
	gint col;
	GtkTreeModel *store;
	GtkTreeIter iter;

	col = GPOINTER_TO_INT (g_object_get_data(G_OBJECT (cellrenderertext), "col"));
	store = gtk_tree_view_get_model (GTK_TREE_VIEW (treeview));
	gtk_tree_model_get_iter_from_string (store, &iter, path);
	gtk_list_store_set (GTK_LIST_STORE(store), &iter, col, text, -1);
	if (col > 0) {
		col = (col == 1) ? 2 : 1;
		gtk_list_store_set (GTK_LIST_STORE(store), &iter, col, "", -1);
	}
	
	window = g_object_get_data(G_OBJECT(treeview), "parent");
	diario_predefinidos_editar_guardar_cambios (window);
}

void
diario_predefinidos_editar_question	(GtkCellRendererText * cellrenderertext,
		      						gchar * path,
									gchar * text,
									gpointer treeview)
{
	GtkWidget *window;
	GtkComboBox *combobox;
	GtkTreeModel *store, *store1;
	GtkTreeIter iter;
	GList *l, *node;
	gchar *name;
	tpredefinido *pred;
	tsimb *ts;

	store = gtk_tree_view_get_model (GTK_TREE_VIEW (treeview));
	gtk_tree_model_get_iter_from_string (store, &iter, path);
	gtk_list_store_set (GTK_LIST_STORE(store), &iter, 1, text, -1);

	// Guardar pregunta
	window = g_object_get_data(G_OBJECT(treeview), "parent");
	combobox = g_object_get_data(G_OBJECT(window), "combobox1");
	store1 = gtk_combo_box_get_model(combobox);
	gtk_combo_box_get_active_iter(combobox, &iter);
	gtk_tree_model_get(store1, &iter, 0, &name, -1);
	l = g_object_get_data(G_OBJECT(store1), "lista");
	for(node = g_list_first(l); node != NULL; node = g_list_next(node)) {
		pred = (tpredefinido *) node->data;
		if (g_ascii_strcasecmp(pred->nombre, name) == 0) {
			g_free(name);
			gtk_tree_model_get_iter_from_string (store, &iter, path);
			gtk_tree_model_get(store, &iter, 0, &name, -1);
			// buscar la variable q es
			for (node = g_list_first(pred->ts); node != NULL; node = g_list_next(node)) {
				ts = (tsimb *) node->data;
				if (strcmp(ts->var, name) == 0) {
					if (ts->question != NULL) g_free(ts->question);
					ts->question = g_strdup(text);
				}
			}
			break;
		}
	}
	g_free(name);

}



void
diario_predefinidos_editar_save (GtkButton *b, gpointer window)
{
	GtkWidget *dialog;
    xmlTextWriterPtr writer;
	GtkComboBox *combobox;
	GtkTreeModel *store;
	GList *l, *node, *node2;
	gchar *filename;
	tpredefinido *pred;
	tsimb *ts;
	tpartidas *tp;
	
	combobox = g_object_get_data(G_OBJECT(window), "combobox1");
	store = gtk_combo_box_get_model(combobox);
	l = g_object_get_data(G_OBJECT(store), "lista");
	
	LIBXML_TEST_VERSION
	
	filename = gconta_uri("predefinidos.xml");
	writer = xmlNewTextWriterFilename(filename, 0);
	g_free(filename);
	if (writer == NULL) {
		dialog = create_dialog("No de pudo guardar los predefinidos", TRUE);
		gtk_dialog_run(GTK_DIALOG(dialog));
		gtk_widget_destroy(dialog);
		return;
	}
	xmlTextWriterStartDocument(writer, NULL, "UTF-8", NULL);
	xmlTextWriterStartDTD(writer, "predefinidos", NULL, NULL);
	xmlTextWriterWriteString(writer, 
		"<!ELEMENT predefinidos (predefinido)*>"
		"<!ELEMENT predefinido (partidas, variables?)>"
		"<!ELEMENT partidas (partida+)>"
		"<!ELEMENT variables (variable+)>"
		"<!ELEMENT partida EMPTY>"
		"<!ELEMENT variable EMPTY>"
		"<!ATTLIST predefinido name CDATA #REQUIRED>"
		"<!ATTLIST partida cuenta CDATA #REQUIRED cuantia CDATA #REQUIRED "
		"tipo (debe|haber) #REQUIRED>"
		"<!ATTLIST variable name CDATA #REQUIRED question CDATA #REQUIRED>");
	xmlTextWriterEndDTD(writer);
    xmlTextWriterStartElement(writer, "predefinidos");
	
	for(node = g_list_first(l); node != NULL; node = g_list_next(node)) {
		pred = (tpredefinido *) node->data;
		if (strlen(pred->nombre) == 0) continue;
		xmlTextWriterStartElement(writer, "predefinido");
		xmlTextWriterWriteAttribute(writer, "name", pred->nombre);

		if (pred->tp != NULL) {
			xmlTextWriterStartElement(writer, "partidas");
			for (node2 = g_list_first(pred->tp); node2 != NULL; node2 = g_list_next(node2)) {
				tp = (tpartidas *) node2->data;
				xmlTextWriterStartElement(writer, "partida");
				xmlTextWriterWriteAttribute(writer, "cuenta", tp->cuenta);
				xmlTextWriterWriteAttribute(writer, "cuantia", tp->cuantia);
				if (tp->debe) xmlTextWriterWriteAttribute(writer, "tipo", "debe");
				else xmlTextWriterWriteAttribute(writer, "tipo", "haber");
				xmlTextWriterEndElement(writer);
			}
			xmlTextWriterEndElement(writer);
		}
		
		if (pred->ts != NULL) {
			xmlTextWriterStartElement(writer, "variables");
			for (node2 = g_list_first(pred->ts); node2 != NULL; node2 = g_list_next(node2)) {
				ts = (tsimb *) node2->data;
				xmlTextWriterStartElement(writer, "variable");
				xmlTextWriterWriteAttribute(writer, "name", ts->var);
				xmlTextWriterWriteAttribute(writer, "question", ts->question);
				xmlTextWriterEndElement(writer);
			}
			xmlTextWriterEndElement(writer);
		}
		
		xmlTextWriterEndElement(writer);
	}
	xmlTextWriterEndElement(writer);
	xmlTextWriterEndDocument(writer);
    xmlFreeTextWriter(writer);
}

void
diario_predefinidos_editar_check (gpointer window)
{
	GtkComboBox *combobox;
	GtkTreeModel *store;
	GtkTreeIter iter;
	GList *l, *node;
	gchar *name;
	tpredefinido *pred;
	
	combobox = g_object_get_data(G_OBJECT(window), "combobox1");
	store = gtk_combo_box_get_model(combobox);
	gtk_combo_box_get_active_iter(combobox, &iter);
	l = g_object_get_data(G_OBJECT(store), "lista");
	
	gtk_tree_model_get(store, &iter, 0, &name, -1);
	
	for (node = g_list_first(l); node != NULL; node = g_list_next(node)) {
		pred = (tpredefinido *) node->data;
		if (g_ascii_strcasecmp(pred->nombre, name) == 0) {
			gconta_tp = pred->tp;
			gconta_ts = pred->ts;
			if (diario_predefinidos_anasin() != -1) {
				pred->tp = gconta_tp;
				pred->ts = gconta_ts;
			} else g_warning("error anasin");
		}
	}
}

void
diario_predefinidos_editar_guardar_cambios (gpointer window)
{
	GtkComboBox *combobox;
	GtkTreeView *treeview;
	GtkTreeModel *store, *store1, *store2;
	GList *lista, *node, *node2, *aux;
	GtkTreeIter iter;
	gchar *name;
	tpredefinido *pred;
	tpartidas *tp;
	tsimb *ts;
	
	combobox = g_object_get_data(G_OBJECT(window), "combobox1");
	store = gtk_combo_box_get_model(combobox);
	treeview = g_object_get_data(G_OBJECT(window), "treeview1");
	store1 = gtk_tree_view_get_model(treeview);
	treeview = g_object_get_data(G_OBJECT(window), "treeview2");
	store2 = gtk_tree_view_get_model(treeview);
	lista = g_object_get_data(G_OBJECT(store), "lista");
	gtk_combo_box_get_active_iter(combobox, &iter);
	gtk_tree_model_get(store, &iter, 0, &name, -1);
	
	for(node = g_list_first(lista); node != NULL; node = g_list_next(node)) {
		pred = (tpredefinido *) node->data;
		if (g_ascii_strcasecmp(pred->nombre, name) == 0) {
			diario_predefinidos_free_tpartidas(pred->tp);
			diario_predefinidos_free_tsimb(pred->ts);
			pred->tp = NULL;
			pred->ts = NULL;
			gtk_tree_model_get_iter_first(store1, &iter);
			while(gtk_list_store_iter_is_valid(GTK_LIST_STORE(store1), &iter)) {
				tp = g_new(tpartidas, 1);
				gtk_tree_model_get(store1, &iter, 0, &(tp->cuenta), -1);
				gtk_tree_model_get(store1, &iter, 1, &name, -1);
				tp->debe = TRUE;
				if (name == NULL || strlen(name) == 0) {
					if (name != NULL) g_free(name);
					gtk_tree_model_get(store1, &iter, 2, &name, -1);
					tp->debe = FALSE;
				}
				tp->cuantia = name;
				pred->tp = g_list_append(pred->tp, tp);
				gtk_tree_model_iter_next(store1, &iter);
			}
			
			diario_predefinidos_editar_check(window);
			// hay q borrar las q sean constantes
			
			for(node2 = g_list_first(pred->ts); node2 != NULL; node2 = g_list_next(node2)) {
				ts = (tsimb *) node2->data;
				if (g_ascii_strtod(ts->var, NULL) != 0) {
					aux = g_list_previous(node2);
					g_free(ts->var);
					g_free(ts);
					pred->ts = g_list_delete_link(pred->ts, node2);
					if (aux == NULL) aux = g_list_first(pred->ts);
					node2 = aux;
				}
			}
			diario_predefinidos_editar_combo(combobox, window);
		}
	}
}

void
diario_predefinidos_editar_new (GtkButton *b, gpointer window)
{
	GtkWidget *dialog;
	GtkWidget *label;
	GtkWidget *entry;
	GtkComboBox *combobox;
	GtkListStore *store;
	GtkTreeIter iter;
	gchar *text;
	GList *l;
	tpredefinido *pred;
	
	dialog = gtk_dialog_new_with_buttons ("Nombre del nuevo predefinido",
			GTK_WINDOW(window), GTK_DIALOG_DESTROY_WITH_PARENT | GTK_DIALOG_MODAL,
			GTK_STOCK_OK, GTK_RESPONSE_ACCEPT, GTK_STOCK_CANCEL, GTK_RESPONSE_REJECT, NULL);
	gtk_box_set_spacing(GTK_BOX(GTK_DIALOG(dialog)->vbox), 12);
	
	label = gtk_label_new ("Introduzca el nombre del predefinido");
	gtk_widget_show(label);
	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), label, FALSE, TRUE, 0);
	
	entry = gtk_entry_new();
	gtk_widget_show(entry);
	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), entry, TRUE, TRUE, 0);
	
	if (gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_ACCEPT) {
		// Meterlo en el combo
		combobox = g_object_get_data(G_OBJECT(window), "combobox1");
		store = (GtkListStore *) gtk_combo_box_get_model(combobox);
		gtk_list_store_append(store, &iter);
		text = g_strdup(gtk_entry_get_text(GTK_ENTRY(entry)));
		gtk_list_store_set(store, &iter, 0, text, -1);
		gtk_combo_box_set_active_iter(combobox, &iter);

		// Meterlo en la lista
		l = g_object_get_data(G_OBJECT(store), "lista");
		pred = g_new(tpredefinido, 1);
		pred->nombre = text;
		pred->ts = NULL;
		pred->tp = NULL;
		l = g_list_append(l, pred);
		g_object_set_data(G_OBJECT(store), "lista", l);
	}
	
	gtk_widget_destroy(dialog);
}

void
diario_predefinidos_editar_eliminar (GtkButton *b, gpointer window)
{
	GtkWidget *dialog;
	GtkComboBox *combobox;
	GtkTreeModel *store;
	GtkTreeIter iter;
	gchar *name;
	GList *l, *node;
	tpredefinido *pred;
	gint val;
	
	// buscar el activo y eliminarlo
	combobox = g_object_get_data(G_OBJECT(window), "combobox1");
	store = gtk_combo_box_get_model(combobox);
	gtk_combo_box_get_active_iter(combobox, &iter);
	if (!gtk_list_store_iter_is_valid(GTK_LIST_STORE(store), &iter)) return;
	gtk_tree_model_get(store, &iter, 0, &name, -1);
	if (name == NULL || strlen(name) == 0) return;
	l = g_object_get_data(G_OBJECT(store), "lista");
	for (node = g_list_first(l); node != NULL; node = g_list_next(node)) {
		pred = (tpredefinido *) node->data;
		if (strcmp (name, pred->nombre) == 0) {
			
			dialog = create_dialog("¿Seguro que desea eliminar el predefinido?", FALSE);
			val = gtk_dialog_run(GTK_DIALOG(dialog));
			gtk_widget_destroy(dialog);
			if (val != GTK_RESPONSE_ACCEPT) {
				g_free(name);
				return;
			}
			
			gtk_list_store_remove(GTK_LIST_STORE(store), &iter);
			g_free(pred->nombre);
			diario_predefinidos_free_tpartidas(pred->tp);
			diario_predefinidos_free_tsimb(pred->ts);
			g_free(pred);
			l = g_list_delete_link(l, node);
			
			break;
		}
	}
	g_free(name);
}
