/*
 * inicio.c
 * This file is part of gconta
 *
 * Copyright (C) 2004 Bruno Orcha Garcia
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, 
 * Boston, MA 02111-1307, USA. 
 */

#include "inicio.h"
#include <sys/stat.h>

void
listar_empresas(GtkListStore *store)
{
	GtkTreeIter iter;
	GDir *dir;
	gchar *path;
	const gchar *l;
	
	path = g_build_filename(g_get_home_dir(), G_DIR_SEPARATOR_S, ".gconta", NULL);
	dir = g_dir_open(path, 0, NULL);
	g_free(path);
	if (dir == NULL) {
		g_warning("No se pudo abrir $HOME/.gconta");
		return;
	}
	for (l = g_dir_read_name(dir); l != NULL; l = g_dir_read_name(dir)) {
		path = xml_empresa_nombre(l);
		if (path != NULL) {
			gtk_list_store_append(store, &iter);
			gtk_list_store_set(store, &iter, 0, path, 1, l, -1); // Guardamos el nombre y el directorio
			g_free(path);
		}
	}
	g_dir_close(dir);
}

GtkWidget*
window_seleccionar_empresa (void)
{
  GtkWidget *window1;
  GtkWidget *vbox1;
  GtkWidget *hbox1;
  GtkWidget *image1;
  GtkWidget *vbox2;
  GtkWidget *label1;
  GtkWidget *scrolledwindow1;
  GtkWidget *treeview1;
  GtkWidget *hbuttonbox1;
  GtkWidget *button1;
  GtkWidget *alignment2;
  GtkWidget *hbox3;
  GtkWidget *image3;
  GtkWidget *label2;
  GtkWidget *button2;
  GtkWidget *alignment1;
  GtkWidget *hbox2;
  GtkWidget *image2;
  GtkWidget *label3;
  GtkListStore *store;
  GtkCellRenderer *rend;
  GtkTreeViewColumn *col;

  window1 = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_container_set_border_width (GTK_CONTAINER (window1), 6);
  gtk_window_set_title (GTK_WINDOW (window1), _("Seleccione empresa"));
  gtk_widget_set_size_request (window1, 500, 300);
  gtk_window_set_resizable (GTK_WINDOW(window1), FALSE);
  
  vbox1 = gtk_vbox_new (FALSE, 12);
  gtk_widget_show (vbox1);
  gtk_container_add (GTK_CONTAINER (window1), vbox1);

  hbox1 = gtk_hbox_new (FALSE, 12);
  gtk_widget_show (hbox1);
  gtk_box_pack_start (GTK_BOX (vbox1), hbox1, TRUE, TRUE, 0);

  image1 = gtk_image_new_from_file (DATA_PREFIX "/empresa.jpg");
  gtk_widget_show (image1);
  gtk_box_pack_start (GTK_BOX (hbox1), image1, FALSE, TRUE, 0);
  gtk_misc_set_alignment (GTK_MISC (image1), 0.5, 0);

  vbox2 = gtk_vbox_new (FALSE, 12);
  gtk_widget_show (vbox2);
  gtk_box_pack_start (GTK_BOX (hbox1), vbox2, TRUE, TRUE, 0);

  label1 = gtk_label_new("<b>Seleccione una empresa</b>");
  gtk_widget_show(label1);
  gtk_label_set_use_markup(GTK_LABEL(label1), TRUE);
  gtk_box_pack_start (GTK_BOX (vbox2), label1, FALSE, FALSE, 0); 

  scrolledwindow1 = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_show (scrolledwindow1);
  gtk_box_pack_start (GTK_BOX (vbox2), scrolledwindow1, TRUE, TRUE, 0);  
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow1), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

  store = gtk_list_store_new(2, G_TYPE_STRING, G_TYPE_STRING);
  listar_empresas(store);
  
  treeview1 = gtk_tree_view_new_with_model(GTK_TREE_MODEL(store));
  gtk_widget_show (treeview1);
  gtk_container_add (GTK_CONTAINER (scrolledwindow1), treeview1);
  gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (treeview1), FALSE);

  rend = gtk_cell_renderer_text_new();
  col = gtk_tree_view_column_new_with_attributes("Cuenta", rend, "text", 0, NULL);
  gtk_tree_view_append_column(GTK_TREE_VIEW(treeview1), col);

  hbuttonbox1 = gtk_hbutton_box_new ();
  gtk_widget_show (hbuttonbox1);
  gtk_box_pack_start (GTK_BOX (vbox1), hbuttonbox1, FALSE, TRUE, 0);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (hbuttonbox1), GTK_BUTTONBOX_END);
  gtk_box_set_spacing (GTK_BOX (hbuttonbox1), 12);

  button1 = gtk_button_new ();
  gtk_widget_show (button1);
  gtk_container_add (GTK_CONTAINER (hbuttonbox1), button1);
  GTK_WIDGET_SET_FLAGS (button1, GTK_CAN_DEFAULT);

  alignment2 = gtk_alignment_new (0.5, 0.5, 0, 0);
  gtk_widget_show (alignment2);
  gtk_container_add (GTK_CONTAINER (button1), alignment2);

  hbox3 = gtk_hbox_new (FALSE, 2);
  gtk_widget_show (hbox3);
  gtk_container_add (GTK_CONTAINER (alignment2), hbox3);

  image3 = gtk_image_new_from_stock ("gtk-new", GTK_ICON_SIZE_BUTTON);
  gtk_widget_show (image3);
  gtk_box_pack_start (GTK_BOX (hbox3), image3, FALSE, FALSE, 0);

  label2 = gtk_label_new_with_mnemonic (_("Nueva empresa"));
  gtk_widget_show (label2);
  gtk_box_pack_start (GTK_BOX (hbox3), label2, FALSE, FALSE, 0);

  button2 = gtk_button_new ();
  gtk_widget_show (button2);
  gtk_container_add (GTK_CONTAINER (hbuttonbox1), button2);
  GTK_WIDGET_SET_FLAGS (button2, GTK_CAN_DEFAULT);

  alignment1 = gtk_alignment_new (0.5, 0.5, 0, 0);
  gtk_widget_show (alignment1);
  gtk_container_add (GTK_CONTAINER (button2), alignment1);

  hbox2 = gtk_hbox_new (FALSE, 2);
  gtk_widget_show (hbox2);
  gtk_container_add (GTK_CONTAINER (alignment1), hbox2);

  image2 = gtk_image_new_from_stock ("gtk-ok", GTK_ICON_SIZE_BUTTON);
  gtk_widget_show (image2);
  gtk_box_pack_start (GTK_BOX (hbox2), image2, FALSE, FALSE, 0);

  label3 = gtk_label_new_with_mnemonic (_("Seleccionar"));
  gtk_widget_show (label3);
  gtk_box_pack_start (GTK_BOX (hbox2), label3, FALSE, FALSE, 0);
  
  g_object_set_data(G_OBJECT(window1), "treeview1", treeview1);
  
  g_signal_connect(button2, "clicked", G_CALLBACK(abrir_empresa), window1);
  g_signal_connect(button1, "clicked", G_CALLBACK(crear_empresa), window1);
  
  return window1;
}

void
abrir_empresa(GtkButton *b, gpointer window1)
{
	GtkWidget *window;
	GtkTreeView *treeview1;
	GtkTreeModel *store;
	GtkTreePath *path;
	GtkTreeIter iter;
	gchar *empresa;
	
	treeview1 = g_object_get_data(G_OBJECT(window1), "treeview1");
	store = gtk_tree_view_get_model(treeview1);
	gtk_tree_view_get_cursor(treeview1, &path, NULL);
	if (path != NULL && gtk_tree_model_get_iter(store, &iter, path)) {
		gtk_tree_model_get(store, &iter, 1, &empresa, -1);
		if (xml_cargar_gconta_main(empresa) == -1) {
			window = create_dialog("No se pudo cargar la empresa", TRUE);
			gtk_dialog_run(GTK_DIALOG(window));
			gtk_widget_destroy(window);
			return;
		}
		g_free(empresa);
		if (gconta_conectar()) {
			window = create_window1();
			gtk_widget_show(window);
			gtk_widget_hide(window1);
		} else {
			window = create_dialog("No se pudo conectar con la base de datos", TRUE);
			gtk_dialog_run(GTK_DIALOG(window));
			gtk_widget_destroy(window);
		}
	}
}

void
crear_empresa	(GtkButton *b, gpointer window)
{
	GtkWidget *w = crear_empresa_ventana(window);
	gtk_widget_show(w);
}


void
crear_empresa_rellenar_combo (GtkWidget *combobox1)
{
	gint n, i;
	GList *node, *ds;
	GdaDataSourceInfo *info;
	
	g_return_if_fail(GTK_IS_COMBO_BOX(combobox1));
	
	// Actualizar el combo
	n = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(combobox1), "tam"));
	for (i = n; i > 0; i--)
		gtk_combo_box_remove_text(GTK_COMBO_BOX(combobox1), i - 1);
	
	ds = gda_config_get_data_source_list();
	for(node = g_list_first(ds); node != NULL; node = g_list_next(node)) {
		info = (GdaDataSourceInfo *) node->data;
		gtk_combo_box_append_text(GTK_COMBO_BOX(combobox1), info->name);
	}
	g_object_set_data(G_OBJECT(combobox1), "tam", GINT_TO_POINTER(g_list_length(ds)));
	gda_config_free_data_source_list(ds);
}


GtkWidget*
crear_empresa_ventana (GtkWindow *window)
{
	GtkWidget *window1;
	GtkWidget *vbox1;
	GtkWidget *table1;
	GtkWidget *label1;
	//GtkWidget *label2;
	GtkWidget *label3;
	GtkWidget *label4;
	GtkWidget *label5;
	GtkWidget *label6;
	GtkWidget *entry1;
	//GtkWidget *entry2;
	GtkWidget *entry3;
	GtkWidget *button4;
	GtkWidget *entry4;
	GtkWidget *combobox1;
	GtkWidget *hbox6;
	GtkWidget *button5;
	GtkWidget *image5;
	GtkWidget *entry5;
	GtkWidget *button6;
	GtkWidget *image6;
	GtkWidget *label7;
	GtkWidget *scrolledwindow1;
	GtkWidget *textview1;
	GtkWidget *hbuttonbox1;
	GtkWidget *button1;
	GtkWidget *alignment2;
	GtkWidget *hbox5;
	GtkWidget *image4;
	GtkWidget *label10;
	GtkWidget *button2;
	GtkWidget *alignment1;
	GtkWidget *hbox4;
	GtkWidget *image3;
	GtkWidget *label9;
	
	window1 = gtk_window_new (GTK_WINDOW_TOPLEVEL);
	gtk_window_set_transient_for(GTK_WINDOW(window1), window);
	gtk_window_set_position(GTK_WINDOW(window1), GTK_WIN_POS_CENTER_ON_PARENT);
	gtk_window_set_modal(GTK_WINDOW(window1), TRUE);
	gtk_container_set_border_width (GTK_CONTAINER (window1), 6);
	gtk_window_set_title (GTK_WINDOW (window1), _("Crear empresa"));

	vbox1 = gtk_vbox_new (FALSE, 12);
	gtk_widget_show (vbox1);
	gtk_container_add (GTK_CONTAINER (window1), vbox1);
	
	table1 = gtk_table_new (5, 2, FALSE);
	gtk_widget_show (table1);
	gtk_box_pack_start (GTK_BOX (vbox1), table1, FALSE, TRUE, 0);
	gtk_table_set_row_spacings (GTK_TABLE (table1), 12);
	gtk_table_set_col_spacings (GTK_TABLE (table1), 12);
	
	label1 = gtk_label_new (_("<b>Nombre de la empresa</b>"));
	gtk_widget_show (label1);
	gtk_table_attach (GTK_TABLE (table1), label1, 0, 1, 0, 1,
					(GtkAttachOptions) (GTK_FILL),
					(GtkAttachOptions) (0), 0, 0);
	gtk_label_set_use_markup (GTK_LABEL (label1), TRUE);
	gtk_misc_set_alignment (GTK_MISC (label1), 0, 0.5);

	entry1 = gtk_entry_new ();
	gtk_widget_show (entry1);
	gtk_table_attach (GTK_TABLE (table1), entry1, 1, 2, 0, 1,
					(GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
					(GtkAttachOptions) (0), 0, 0);

	/*label2 = gtk_label_new (_("Alias"));
	gtk_widget_show (label2);
	gtk_table_attach (GTK_TABLE (table1), label2, 0, 1, 1, 2,
					(GtkAttachOptions) (GTK_FILL),
					(GtkAttachOptions) (0), 0, 0);
	gtk_misc_set_alignment (GTK_MISC (label2), 0, 0.5);

	entry2 = gtk_entry_new ();
	gtk_widget_show (entry2);
	gtk_table_attach (GTK_TABLE (table1), entry2, 1, 2, 1, 2,
					(GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
					(GtkAttachOptions) (0), 0, 0);*/

	label3 = gtk_label_new (_("Año del ejercicio"));
	gtk_widget_show (label3);
	gtk_table_attach (GTK_TABLE (table1), label3, 0, 1, 1, 2,
					(GtkAttachOptions) (GTK_FILL),
					(GtkAttachOptions) (0), 0, 0);
	gtk_misc_set_alignment (GTK_MISC (label3), 0, 0.5);

	entry3 = gtk_entry_new ();
	gtk_widget_show (entry3);
	gtk_table_attach (GTK_TABLE (table1), entry3, 1, 2, 1, 2,
					(GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
					(GtkAttachOptions) (0), 0, 0);

	label4 = gtk_label_new (_("Utilizar plan"));
	gtk_widget_show (label4);
	gtk_table_attach (GTK_TABLE (table1), label4, 0, 1, 2, 3,
					(GtkAttachOptions) (GTK_FILL),
					(GtkAttachOptions) (0), 0, 0);
	gtk_misc_set_alignment (GTK_MISC (label4), 0, 0.5);

	hbox4 = gtk_hbox_new (FALSE, 12);
	gtk_widget_show (hbox4);
	gtk_table_attach (GTK_TABLE (table1), hbox4, 1, 2, 2, 3,
					(GtkAttachOptions) (GTK_FILL),
					(GtkAttachOptions) (GTK_FILL), 0, 0);

	button4 = gtk_button_new ();
	gtk_widget_show (button4);
	gtk_box_pack_start (GTK_BOX (hbox4), button4, FALSE, FALSE, 0);

	image4 = gtk_image_new_from_stock ("gtk-open", GTK_ICON_SIZE_BUTTON);
	gtk_widget_show (image4);
	gtk_container_add (GTK_CONTAINER (button4), image4);

	entry4 = gtk_entry_new ();
	gtk_widget_show (entry4);
	gtk_box_pack_start (GTK_BOX (hbox4), entry4, TRUE, TRUE, 0);

	label5 = gtk_label_new (_("Asiento de apertura"));
	gtk_widget_show (label5);
	gtk_table_attach (GTK_TABLE (table1), label5, 0, 1, 3, 4,
					(GtkAttachOptions) (GTK_FILL),
					(GtkAttachOptions) (0), 0, 0);
	gtk_misc_set_alignment (GTK_MISC (label5), 0, 0.5);

	hbox5 = gtk_hbox_new (FALSE, 12);
	gtk_widget_show (hbox5);
	gtk_table_attach (GTK_TABLE (table1), hbox5, 1, 2, 3, 4,
					(GtkAttachOptions) (GTK_FILL),
					(GtkAttachOptions) (GTK_FILL), 0, 0);

	button5 = gtk_button_new ();
	gtk_widget_show (button5);
	gtk_box_pack_start (GTK_BOX (hbox5), button5, FALSE, FALSE, 0);

	image5 = gtk_image_new_from_stock ("gtk-open", GTK_ICON_SIZE_BUTTON);
	gtk_widget_show (image5);
	gtk_container_add (GTK_CONTAINER (button5), image5);

	entry5 = gtk_entry_new ();
	gtk_widget_show (entry5);
	gtk_box_pack_start (GTK_BOX (hbox5), entry5, TRUE, TRUE, 0);

	label6 = gtk_label_new (_("Fuente de datos"));
	gtk_widget_show (label6);
	gtk_table_attach (GTK_TABLE (table1), label6, 0, 1, 4, 5,
					(GtkAttachOptions) (GTK_FILL),
					(GtkAttachOptions) (0), 0, 0);
	gtk_misc_set_alignment (GTK_MISC (label6), 0, 0.5);

	hbox6 = gtk_hbox_new (FALSE, 12);
	gtk_widget_show (hbox6);
	gtk_table_attach (GTK_TABLE (table1), hbox6, 1, 2, 4, 5,
					(GtkAttachOptions) (GTK_FILL),
					(GtkAttachOptions) (GTK_FILL), 0, 0);

	combobox1 = gtk_combo_box_new_text();
	g_object_set_data(G_OBJECT(combobox1), "tam", GINT_TO_POINTER(0));
	crear_empresa_rellenar_combo (combobox1);

	button6 = gtk_button_new ();
	gtk_widget_show (button6);
	gtk_box_pack_start (GTK_BOX (hbox6), button6, FALSE, FALSE, 0);
	
	image6 = gtk_image_new_from_stock ("gtk-add", GTK_ICON_SIZE_BUTTON);
	gtk_widget_show (image6);
	gtk_container_add (GTK_CONTAINER (button6), image6);
	
	gtk_widget_show (combobox1);
	gtk_box_pack_start (GTK_BOX (hbox6), combobox1, TRUE, TRUE, 0);
	
	label7 = gtk_label_new (_("Descripci\303\263n y datos adicionales"));
	gtk_widget_show (label7);
	gtk_box_pack_start (GTK_BOX (vbox1), label7, FALSE, FALSE, 0);

	scrolledwindow1 = gtk_scrolled_window_new (NULL, NULL);
	gtk_widget_show (scrolledwindow1);
	gtk_box_pack_start (GTK_BOX (vbox1), scrolledwindow1, TRUE, TRUE, 0);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow1), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

	textview1 = gtk_text_view_new ();
	gtk_widget_show (textview1);
	gtk_container_add (GTK_CONTAINER (scrolledwindow1), textview1);
	
	hbuttonbox1 = gtk_hbutton_box_new ();
	gtk_widget_show (hbuttonbox1);
	gtk_box_pack_start (GTK_BOX (vbox1), hbuttonbox1, TRUE, TRUE, 0);
	gtk_button_box_set_layout (GTK_BUTTON_BOX (hbuttonbox1), GTK_BUTTONBOX_END);
	gtk_box_set_spacing (GTK_BOX (hbuttonbox1), 12);
	
	button1 = gtk_button_new ();
	gtk_widget_show (button1);
	gtk_container_add (GTK_CONTAINER (hbuttonbox1), button1);
	GTK_WIDGET_SET_FLAGS (button1, GTK_CAN_DEFAULT);
	
	alignment2 = gtk_alignment_new (0.5, 0.5, 0, 0);
	gtk_widget_show (alignment2);
	gtk_container_add (GTK_CONTAINER (button1), alignment2);
	
	hbox5 = gtk_hbox_new (FALSE, 2);
	gtk_widget_show (hbox5);
	gtk_container_add (GTK_CONTAINER (alignment2), hbox5);
	
	image4 = gtk_image_new_from_stock ("gtk-cancel", GTK_ICON_SIZE_BUTTON);
	gtk_widget_show (image4);
	gtk_box_pack_start (GTK_BOX (hbox5), image4, FALSE, FALSE, 0);
	
	label10 = gtk_label_new_with_mnemonic (_("Cancelar"));
	gtk_widget_show (label10);
	gtk_box_pack_start (GTK_BOX (hbox5), label10, FALSE, FALSE, 0);
	
	button2 = gtk_button_new ();
	gtk_widget_show (button2);
	gtk_container_add (GTK_CONTAINER (hbuttonbox1), button2);
	GTK_WIDGET_SET_FLAGS (button2, GTK_CAN_DEFAULT);
	
	alignment1 = gtk_alignment_new (0.5, 0.5, 0, 0);
	gtk_widget_show (alignment1);
	gtk_container_add (GTK_CONTAINER (button2), alignment1);
	
	hbox4 = gtk_hbox_new (FALSE, 2);
	gtk_widget_show (hbox4);
	gtk_container_add (GTK_CONTAINER (alignment1), hbox4);
	
	image3 = gtk_image_new_from_stock ("gtk-ok", GTK_ICON_SIZE_BUTTON);
	gtk_widget_show (image3);
	gtk_box_pack_start (GTK_BOX (hbox4), image3, FALSE, FALSE, 0);
	
	label9 = gtk_label_new_with_mnemonic (_("Crear"));
	gtk_widget_show (label9);
	gtk_box_pack_start (GTK_BOX (hbox4), label9, FALSE, FALSE, 0);
	
	g_object_set_data(G_OBJECT(window1), "parent", window); // name
	g_object_set_data(G_OBJECT(window1), "entry1", entry1); // name
	//g_object_set_data(G_OBJECT(window1), "entry2", entry2); // alias = gdasource
	g_object_set_data(G_OBJECT(window1), "entry3", entry3); // año
	g_object_set_data(G_OBJECT(window1), "entry4", entry4); // pgc.xml
	g_object_set_data(G_OBJECT(window1), "entry5", entry5); // asiento_apertura.xml
	g_object_set_data(G_OBJECT(window1), "textview1", textview1); // Descripcion
	g_object_set_data(G_OBJECT(window1), "combobox1", combobox1); // Combo

	g_signal_connect (button1, "clicked", G_CALLBACK (crear_empresa_cancelar), window1);
	g_signal_connect (button2, "clicked", G_CALLBACK (crear_empresa_aceptar), window1);
	g_signal_connect (button4, "clicked", G_CALLBACK (seleccionar_fichero), entry4);
	g_signal_connect (button5, "clicked", G_CALLBACK (seleccionar_fichero), entry5);
	g_signal_connect (button6, "clicked", G_CALLBACK (asistente_gda), window1);

	return window1;
}

void
seleccionar_fichero(GtkButton *b, GtkEntry *entry)
{
	GtkWidget *dialog;
	GtkWidget *parent;
	gchar *uri;
	
	parent = gtk_widget_get_toplevel(GTK_WIDGET(entry));
	dialog = gtk_file_chooser_dialog_new("Selecciona un archivo", GTK_WINDOW(parent),
			GTK_FILE_CHOOSER_ACTION_OPEN,
			GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
			GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
			NULL);
	if (gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_ACCEPT) {
		uri = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(dialog));
		gtk_entry_set_text(entry, uri);
	}
	gtk_widget_destroy(dialog);
}

void
crear_empresa_cancelar(GtkButton *b, GtkWidget *window)
{
	gtk_widget_destroy(window);
}

void
crear_empresa_aceptar(GtkButton *b, GtkWidget *window)
{
    xmlTextWriterPtr writer;
	GtkWidget *dialog;
	GtkWidget *entry;
	GtkWidget *combobox1;
	GtkWidget *textview1;
	GtkWidget *parent;
	GtkWidget *treeview1;
	GtkTextBuffer *textbuffer;
	GtkTextIter iter1, iter2;
	const gchar *nombre;
	const gchar *alias;
	gchar *gdasource;
	gchar *descripcion;
	gchar *buffer;
	gchar *ruta;
	gchar *filename;
	const gchar *buffer2;
	GList *ds, *node;
	GdaDataSourceInfo *dsi;
	gint i, year;
	GdaTransaction *trans;
		
	entry = g_object_get_data(G_OBJECT(window), "entry1");
	nombre = gtk_entry_get_text(GTK_ENTRY(entry));
	if (g_utf8_strlen(nombre, -1) == 0) {
		dialog = create_dialog (_("Escriba un nombre para la empresa"), TRUE);
		gtk_dialog_run (GTK_DIALOG (dialog));
		gtk_widget_destroy (dialog);
		return;
	}
	
	entry = g_object_get_data(G_OBJECT(window), "entry3");
	buffer2 = gtk_entry_get_text(GTK_ENTRY(entry));
	year = g_ascii_strtod(buffer2, NULL);
	if (year < 1900 || year > 2100){
		dialog = create_dialog ("El año introducido no es válido", TRUE);
		gtk_dialog_run (GTK_DIALOG (dialog));
		gtk_widget_destroy (dialog);
		return;
	}
	
	textview1 = g_object_get_data(G_OBJECT(window), "textview1");
	textbuffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(textview1));
	gtk_text_buffer_get_bounds(textbuffer, &iter1, &iter2);
	descripcion = gtk_text_buffer_get_text(textbuffer, &iter1, &iter2, TRUE);
	
	ds = gda_config_get_data_source_list();
	combobox1 = g_object_get_data(G_OBJECT(window), "combobox1");
	i = gtk_combo_box_get_active(GTK_COMBO_BOX(combobox1));
	node = g_list_nth(ds, i);
	if (node == NULL) {
		dialog = create_dialog (_("Seleccione una fuente de datos para su empresa"), TRUE);
		gtk_dialog_run (GTK_DIALOG (dialog));
		gtk_widget_destroy (dialog);
		return;
	}
	dsi = (GdaDataSourceInfo *) node->data;
	gdasource = g_strdup(dsi->name);
	alias = gdasource;
	gda_config_free_data_source_list(ds);

	// Crear tablas
	gconta_main.alias = (gchar *) alias;
	gconta_main.gda_source = gdasource;
	if (gconta_conectar()) {
		if (!gda_connection_supports(gconta_main.conn,
			GDA_CONNECTION_FEATURE_TRANSACTIONS)) {
			dialog = create_dialog ("El SGBD no soporta transacciones!!", TRUE);
			gtk_dialog_run (GTK_DIALOG (dialog));
			gtk_widget_destroy (dialog);
			return;
		}
		
		/* comienza transaccion */
		trans = gda_transaction_new ("cuenta");
		gda_transaction_set_isolation_level (trans,
							 GDA_TRANSACTION_ISOLATION_SERIALIZABLE);
		if (!gda_connection_begin_transaction (gconta_main.conn, trans)) {
			gconta_errores ();
			dialog = create_dialog ("No se pudo comenzar la transacción", TRUE);
			gtk_dialog_run (GTK_DIALOG (dialog));
			gtk_widget_destroy (dialog);
			return;
		}
		
		// PGC
		buffer = g_strdup_printf (
			"create table PGC%d ("
			"	cuenta int,"
			"	nombre text not null,"
			"	padre int references PGC%d (cuenta),"
			"	primary key (cuenta));", year, year);
		GCONTA_nonquery(buffer, "Error al crear PGC", trans);
		g_free(buffer);
		
		// PGC_DESCRIPCION
		buffer = g_strdup_printf (
			"create table PGC_DESCRIPCION%d ("
        	"	cuenta int references PGC%d (cuenta),"
        	"	descripcion text not null,"
        	"	primary key (cuenta));", year, year);
		GCONTA_nonquery(buffer, "Error al crear PGC_DESCRIPCION", trans);
		g_free(buffer);
		
		// DIARIO
		buffer = g_strdup_printf (
			"create table DIARIO%d ("
			"	asiento int,"
			"	fecha date not null,"
			"	primary key (asiento));", year);
		GCONTA_nonquery(buffer, "Error al crear DIARIO", trans);
		g_free(buffer);
		
		// DIARIO_DESCRIPCION
		buffer = g_strdup_printf (
			"create table DIARIO_DESCRIPCION%d ("
			"asiento int references DIARIO%d (asiento),"
			"descripcion text not null,"
			"primary key (asiento));", year, year);
		GCONTA_nonquery(buffer, "Error al crear DIARIO_DESCRIPCION", trans);
		g_free(buffer);
		
		// DIARIO_DEBE
		buffer = g_strdup_printf (
			"create table DIARIO_DEBE%d ("
			"asiento int references DIARIO%d (asiento),"
			"cuenta int references PGC%d (cuenta),"
			"cuantia decimal not null,"
			"primary key (asiento, cuenta));", year, year, year);
		GCONTA_nonquery(buffer, "Error al crear DIARIO_DEBE", trans);
		g_free(buffer);
		
		// DIARIO_HABER
		buffer = g_strdup_printf (
			"create table DIARIO_HABER%d ("
			"asiento int references DIARIO%d (asiento),"
			"cuenta int references PGC%d (cuenta),"
			"cuantia decimal not null,"
			"primary key (asiento, cuenta));", year, year, year);
		GCONTA_nonquery(buffer, "Error al crear DIARIO_HABER", trans);
		g_free(buffer);
		
		// CIERRE
		buffer = g_strdup_printf (
			"create table CIERRE%d ("
			"cuenta int references PGC%d (cuenta),"
			"debe decimal,"
			"haber decimal,"
			"primary key (cuenta));", year, year);
		GCONTA_nonquery(buffer, "Error al crear PGC", trans);
		g_free(buffer);
		
		// Fin de transaccion
		gda_connection_commit_transaction (gconta_main.conn, trans);
		g_object_unref (trans);
	} else {
		dialog = create_dialog("No se pudo establecer la conexion con la base de datos", TRUE);
		gtk_dialog_run(GTK_DIALOG(dialog));
		gtk_widget_destroy(dialog);
		return;
	}

	// Crear carpeta y copiar archivos basicos
	buffer = g_strdup_printf("%s" G_DIR_SEPARATOR_S ".gconta", g_get_home_dir());
	if (!g_file_test(buffer, G_FILE_TEST_IS_DIR))
		if (mkdir(buffer, S_IRUSR | S_IWUSR | S_IXUSR) != 0) g_error("No se pudo crear .gconta");
	g_free(buffer);
	
	ruta = g_strdup_printf("%s" G_DIR_SEPARATOR_S ".gconta" G_DIR_SEPARATOR_S "%s%d", g_get_home_dir(), alias, year);
	if (!g_file_test(ruta, G_FILE_TEST_IS_DIR))
		if (mkdir(ruta, S_IRUSR | S_IWUSR | S_IXUSR) != 0) g_error("No se pudo crear la carpeta para la empresa");
	
	//  Realizar copia asincrona, uso de gthread
	//	tar -C default -xf DATA_PREFIX "/default.tar"
	buffer = g_strdup_printf("tar -C %s -xf " DATA_PREFIX "/default.tar", ruta);
	system(buffer);
	g_free(buffer);
		
	// Crear empresa.xml con los datos de la empresa
	filename = g_build_filename(ruta, G_DIR_SEPARATOR_S, "empresa.xml", NULL);
	writer = xmlNewTextWriterFilename(filename, 0);
	g_free(filename);
	if (writer == NULL) {
		dialog = create_dialog("No se pudo crear correctamente la empresa", TRUE);
		gtk_dialog_run(GTK_DIALOG(dialog));
		gtk_widget_destroy(dialog);
		return;
	}
	xmlTextWriterStartDocument(writer, NULL, "UTF-8", NULL);
	xmlTextWriterStartDTD(writer, "empresa", NULL, NULL);
	xmlTextWriterWriteString(writer, "<!ELEMENT empresa (nombre, alias,"
		" gdasource, descripcion?, ejercicio)><!ELEMENT nombre (#PCDATA)><!ELEMENT alias "
		"(#PCDATA)><!ELEMENT gdasource (#PCDATA)><!ELEMENT descripcion (#PCDATA)><!ELEMENT "
		"ejercicio (#PCDATA)>");
	xmlTextWriterEndDTD(writer);
    xmlTextWriterStartElement(writer, "empresa");
	xmlTextWriterWriteElement(writer, "nombre", nombre);
	xmlTextWriterWriteElement(writer, "alias", alias);
	xmlTextWriterWriteElement(writer, "gdasource", gdasource);	
	xmlTextWriterWriteElement(writer, "descripcion", descripcion);
	
	buffer = g_strdup_printf("%d", year);
	xmlTextWriterWriteElement(writer, "ejercicio", buffer);
	g_free(buffer);
	
	xmlTextWriterEndElement(writer);
	xmlTextWriterEndDocument(writer);
    xmlFreeTextWriter(writer);	
	
	// Copiar al directorio de la empresa el pgc.xml a importar
	// Eliminar las llamadas system!!
	entry = g_object_get_data(G_OBJECT(window), "entry4"); // Pgc origen
	filename = (gchar *) gtk_entry_get_text(GTK_ENTRY(entry));
	if (filename != NULL && strlen(filename) != 0) {
		buffer = g_strdup_printf("cp -f %s %s/pgc.xml", filename, ruta);
		system(buffer);
		g_free(buffer);
	}
	xml_cargar_pgc(year);
	
	entry = g_object_get_data(G_OBJECT(window), "entry5"); // asiento apertura
	filename = (gchar *) gtk_entry_get_text(GTK_ENTRY(entry));
	if (filename != NULL) ; //xml_cargar_asiento_apertura(filename, year);
	
	parent = g_object_get_data(G_OBJECT(window), "parent");
	treeview1 = g_object_get_data(G_OBJECT(parent), "treeview1");
	gtk_list_store_clear(GTK_LIST_STORE(gtk_tree_view_get_model(GTK_TREE_VIEW(treeview1))));
	listar_empresas(GTK_LIST_STORE(gtk_tree_view_get_model(GTK_TREE_VIEW(treeview1))));
	gtk_widget_destroy(window);
}


void
asistente_gda_finished	(GnomeDbDsnConfigDruid *druid,
						gboolean error,
						gpointer win)
{
const GdaDataSourceInfo *dsninfo;
GtkWidget *combobox1;
GtkWidget *parent;
	
parent = g_object_get_data(G_OBJECT(win), "parent");
combobox1 = g_object_get_data(G_OBJECT(parent), "combobox1");

if (!error) {
	dsninfo = gnome_db_dsn_config_druid_get_dsn (druid);
	if (dsninfo) {
		gda_config_save_data_source (dsninfo->name,
			  dsninfo->provider,
			  dsninfo->cnc_string,
			  dsninfo->description,
			  dsninfo->username,
			  dsninfo->password);
		
		crear_empresa_rellenar_combo (combobox1);
		
	} else gnome_db_show_error (_("No se pudo crear una fuente de datos"));
}
	gtk_widget_destroy (GTK_WIDGET (win));
}

void
asistente_gda 	(GtkButton *b,
				gpointer parent)
{
	GtkWidget *win;
	GtkWidget *druid;
	
	win = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_window_set_transient_for(GTK_WINDOW(win), GTK_WINDOW(parent));
	gtk_window_set_modal(GTK_WINDOW(win), TRUE);
	gtk_window_set_position(GTK_WINDOW(win), GTK_WIN_POS_CENTER_ON_PARENT);
	
	druid = gnome_db_dsn_config_druid_new();
	gtk_widget_show(druid);
	gtk_container_add(GTK_CONTAINER(win), druid);
	
	gtk_widget_show(win);
	
	g_object_set_data(G_OBJECT(win), "parent", parent);
	
	g_signal_connect (G_OBJECT(druid), "finished", G_CALLBACK(asistente_gda_finished), win);
}
