/*
 * notas.c
 * This file is part of gconta
 *
 * Copyright (C) 2004 Bruno Orcha Garcia
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, 
 * Boston, MA 02111-1307, USA. 
 */

#include "notas.h"


GtkWidget *
notas_barra (gpointer *window)
{
	GtkWidget *toolbar1;
	GtkWidget *toolbutton1;
	GtkWidget *separatortoolitem1;
	GtkWidget *toolbuttoncerrar;
	gint tmp_toolbar_icon_size;
	GtkWidget *tmp_image;

	toolbar1 = gtk_toolbar_new ();
	gtk_widget_show (toolbar1);
	gtk_toolbar_set_style (GTK_TOOLBAR (toolbar1), GTK_TOOLBAR_ICONS);
	tmp_toolbar_icon_size =	gtk_toolbar_get_icon_size (GTK_TOOLBAR (toolbar1));

	tmp_image =
		gtk_image_new_from_stock ("gtk-save", tmp_toolbar_icon_size);
	gtk_widget_show (tmp_image);
	toolbutton1 = (GtkWidget *) gtk_tool_button_new (tmp_image, "");
	gtk_widget_show (toolbutton1);
	gtk_container_add (GTK_CONTAINER (toolbar1), toolbutton1);
	gtk_tool_item_set_is_important (GTK_TOOL_ITEM (toolbutton1), TRUE);

	separatortoolitem1 = (GtkWidget *) gtk_separator_tool_item_new ();
	gtk_widget_show (separatortoolitem1);
	gtk_tool_item_set_expand (GTK_TOOL_ITEM (separatortoolitem1), TRUE);
	gtk_container_add (GTK_CONTAINER (toolbar1), separatortoolitem1);
	gtk_separator_tool_item_set_draw (GTK_SEPARATOR_TOOL_ITEM
					  (separatortoolitem1), FALSE);

	tmp_image =
		gtk_image_new_from_stock ("gtk-close", tmp_toolbar_icon_size);
	gtk_widget_show (tmp_image);
	toolbuttoncerrar = (GtkWidget *) gtk_tool_button_new (tmp_image, "");
	gtk_widget_show (toolbuttoncerrar);
	gtk_container_add (GTK_CONTAINER (toolbar1), toolbuttoncerrar);

	g_signal_connect(toolbutton1, "clicked",
			G_CALLBACK(notas_guardar), window);
	g_signal_connect((gpointer) toolbuttoncerrar, "clicked",
			G_CALLBACK(notas_barra_cerrar), window);

	return toolbar1;
}

GtkWidget *
notas_child (gpointer window)
{
	GtkWidget *scrolledwindow1;
	GtkWidget *textview1;

	scrolledwindow1 = gtk_scrolled_window_new (NULL, NULL);
	gtk_widget_show (scrolledwindow1);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow1),
					GTK_POLICY_AUTOMATIC,
					GTK_POLICY_AUTOMATIC);

	textview1 = gtk_text_view_new ();
	gtk_widget_show (textview1);
	gtk_container_add (GTK_CONTAINER (scrolledwindow1), textview1);

	notas_cargar(textview1);
	
	g_object_set_data (G_OBJECT (scrolledwindow1), "textview1", textview1);

	g_object_set_data(G_OBJECT(scrolledwindow1), "modificado", GINT_TO_POINTER(0));
	
	g_signal_connect(gtk_text_view_get_buffer(GTK_TEXT_VIEW(textview1)), "changed",
			G_CALLBACK(notas_cambiado), window);
		
	return scrolledwindow1;
}

void
notas_barra_cerrar	(GtkToolButton * b,
					 gpointer window)
{
	GtkWidget *child;
	GtkWidget *dialog;
	gint t;
	
	child = gconta_get_child(window);
	t = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(child), "modificado"));
	
	if (t != 0) {
		dialog = create_dialog("Ha abido cambios en las notas.\n¿Desea guardar los cambios?", FALSE);
		if (gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_ACCEPT) {
			notas_guardar(NULL, window);
		}
		gtk_widget_destroy(dialog);
	}
	
	gconta_barra_cerrar(window);
}

void
notas_cambiado (GtkTextBuffer *b,
				gpointer window)
{
	GtkWidget *child;
	
	child = gconta_get_child(window);
	g_object_set_data(G_OBJECT(child), "modificado", GINT_TO_POINTER(1));
}

void
notas_guardar	(GtkToolButton *b,
				gpointer window)
{
	GtkWidget *child;
	GtkWidget *textview1;
	GtkWidget *dialog;
	GIOChannel *file;
	GtkTextBuffer *buffer;
	GtkTextIter iter1, iter2;
	gchar *t, *filename;
	gint size;

	child = gconta_get_child(window);
	textview1 = g_object_get_data(G_OBJECT(child), "textview1");
	buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(textview1));
	gtk_text_buffer_get_bounds(buffer, &iter1, &iter2);
	
	filename = gconta_uri("notas.txt");
	file = g_io_channel_new_file (filename, "w+", NULL);
	g_free(filename);
	if (file == NULL) {
		dialog = create_dialog("Error al abrir notas.txt", TRUE);
		gtk_dialog_run(GTK_DIALOG(dialog));
		gtk_widget_destroy(dialog);
		return;
	}
	t = gtk_text_buffer_get_text(buffer, &iter1, &iter2, FALSE);
	if (g_io_channel_write_chars(file, t, strlen(t), &size, NULL) != G_IO_STATUS_NORMAL)
	{
		dialog = create_dialog("Error al guardar notas.txt", TRUE);
		gtk_dialog_run(GTK_DIALOG(dialog));
		gtk_widget_destroy(dialog);
		g_free(t);
		return;
	}
	g_free(t);
	g_io_channel_close(file);
	g_object_set_data(G_OBJECT(child), "modificado", GINT_TO_POINTER(0));
}

void
notas_cargar (GtkWidget *textview1)
{
	GtkWidget *dialog;
	GIOChannel *file;
	GtkTextBuffer *buffer;
	gchar *s, *filename;
	gint size;
	
	buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(textview1));
	filename = gconta_uri("notas.txt");
	file = g_io_channel_new_file (filename, "r", NULL);
	g_free(filename);
	if (file == NULL) {
		dialog = create_dialog("Error al abrir notas.txt", TRUE);
		gtk_dialog_run(GTK_DIALOG(dialog));
		gtk_widget_destroy(dialog);
		return;
	}
	if (g_io_channel_read_to_end(file, &s, &size, NULL) != G_IO_STATUS_NORMAL) {
		dialog = create_dialog("Error al leer notas.txt", TRUE);
		gtk_dialog_run(GTK_DIALOG(dialog));
		gtk_widget_destroy(dialog);
		return;
	}
	
	gtk_text_buffer_set_text(buffer, s, size);
	g_free(s);
	g_io_channel_close(file);
}
