/*
 * varios.c
 * This file is part of gconta
 *
 * Copyright (C) 2004 Bruno Orcha Garcia
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, 
 * Boston, MA 02111-1307, USA. 
 */

#include "varios.h"

void
asiento_cierre (GtkMenuItem *mi, gpointer window)
{
	GdaCommand *cmd;
	GdaDataModel *data;
	GdaValue *v;
	GList *l, *node;
	GtkWidget *child;
	GtkWidget *textview;
	GtkTextBuffer *textbuffer;
	GtkListStore *store;
	GtkTreeView *treeview;
	GtkTreeIter iter;
	GtkTreePath *path, *path1;
	GtkTreeViewColumn *col;
	gint i, j;
	gdouble valor;
	gchar *cuenta, *cuantia;
	gchar buffer[MAX_BUFFER];
	gchar bufferfecha1[11];
	gchar bufferfecha2[11];
	gchar *text_path, *filename;
    xmlTextWriterPtr writer;

	g_return_if_fail(gconta_conectado());
	
	filename = gconta_uri("asiento_apertura.xml");
	if (g_file_test(filename, G_FILE_TEST_EXISTS)) {
		child = create_dialog("Parece ser que ya ha creado un asiento de cierre.\n"
							  "Para crear otro debe eliminar el asiento de apertura\n"
							  "existente en la carpeta de la empresa.", TRUE);
		gtk_dialog_run(GTK_DIALOG(child));
		gtk_widget_destroy(child);
		g_free(filename);
		return;
	}
	g_free(filename);
	
	g_snprintf(buffer, MAX_BUFFER, "DELETE FROM CIERRE%d", gconta_main.year);
	cmd = gda_command_new(buffer, GDA_COMMAND_TYPE_SQL, GDA_COMMAND_OPTION_STOP_ON_ERRORS);
	i = gda_connection_execute_non_query(gconta_main.conn, cmd, NULL);
	gda_command_free(cmd);
	if (i == -1) {
		gconta_errores();
		return;
	}
	
	g_snprintf(buffer, MAX_BUFFER, "SELECT cuenta FROM DIARIO_DEBE%d UNION SELECT cuenta FROM DIARIO_HABER%d;"
			"SELECT cuenta, SUM(cuantia) FROM DIARIO_DEBE%d GROUP BY cuenta;"
			"SELECT cuenta, SUM(cuantia) FROM DIARIO_HABER%d GROUP BY cuenta;",
			gconta_main.year, gconta_main.year, gconta_main.year, gconta_main.year);
	cmd = gda_command_new(buffer, GDA_COMMAND_TYPE_SQL, GDA_COMMAND_OPTION_STOP_ON_ERRORS);
	l = gda_connection_execute_command(gconta_main.conn, cmd, NULL);
	gda_command_free(cmd);
	node = g_list_first(l);
	for(j = 0; j < 3; j++) {
		data = (GdaDataModel *) node->data;
		if (data == NULL) {
			gconta_errores();
			return;
		}
		for(i = 0; i < gda_data_model_get_n_rows(data); i++) {
			v = (GdaValue *) gda_data_model_get_value_at(data, 0, i);
			cuenta = gda_value_stringify(v);
			if (j != 0) {
				v = (GdaValue *) gda_data_model_get_value_at(data, 1, i);
				cuantia = gda_value_stringify(v);
			}
			switch(j) {
			case 0:
				g_snprintf(buffer, MAX_BUFFER, "INSERT INTO CIERRE%d(cuenta, debe, haber) VALUES (%s, 0, 0)",
						gconta_main.year, cuenta);
				break;
			case 1:
				g_snprintf(buffer, MAX_BUFFER, "UPDATE CIERRE%d SET debe = %s WHERE cuenta = %s",
						gconta_main.year, cuantia, cuenta);
				break;
			case 2:
				g_snprintf(buffer, MAX_BUFFER, "UPDATE CIERRE%d SET haber = %s WHERE cuenta = %s",
						gconta_main.year, cuantia, cuenta);
				break;
			}
			GCONTA_nonquery(buffer, "Error al crear asiento de cierre", NULL);
			g_free(cuenta);
			if (j != 0) g_free(cuantia);
		}
		g_object_unref(G_OBJECT(data));
		node = g_list_next(node);
	}
	g_list_free(l);
	// Tabla CIERRE correctamente creada
	
	g_snprintf(buffer, MAX_BUFFER, "SELECT cuenta, haber - debe FROM CIERRE%d WHERE debe <> haber", gconta_main.year);
	cmd = gda_command_new(buffer, GDA_COMMAND_TYPE_SQL, GDA_COMMAND_OPTION_STOP_ON_ERRORS);
	data = gda_connection_execute_single_command(gconta_main.conn, cmd, NULL);
	gda_command_free(cmd);
	if (data == NULL) {
		gconta_errores();
		return;
	}
	
	// ahora hay que crear el asiento
	boton_diario(NULL, window);
	child = gconta_get_child(window);
	treeview = g_object_get_data (G_OBJECT (child), "treeview");
	store = (GtkListStore *) gtk_tree_view_get_model (treeview);
	diario_barra_limpiar (NULL, window);

	// Crear asiento_apertura.xml
	filename = gconta_uri("asiento_apertura.xml");
	writer = xmlNewTextWriterFilename(filename, 0);
	g_free(filename);
	if (writer == NULL) {
		child = create_dialog("No se pudo crear asiento_apertura.xml", TRUE);
		gtk_dialog_run(GTK_DIALOG(child));
		gtk_widget_destroy(child);
		return;
	}
	xmlTextWriterStartDocument(writer, NULL, "UTF-8", NULL);
	xmlTextWriterStartDTD(writer, "asiento_apertura", NULL, NULL);
	xmlTextWriterWriteDTDElement(writer, "asiento_apertura", "(partida, partida+)");
	xmlTextWriterWriteDTDElement(writer, "partida", "EMPTY");
	xmlTextWriterWriteString(writer, "<!ATTLIST partida cuenta CDATA #REQUIRED cuantia CDATA #REQUIRED>");
	xmlTextWriterEndDTD(writer);
    xmlTextWriterStartElement(writer, "asiento_apertura");

	for(i = 0; i < gda_data_model_get_n_rows(data); i++) {
		v = (GdaValue *) gda_data_model_get_value_at(data, 0, i);
		cuenta = gda_value_stringify(v);
		v = (GdaValue *) gda_data_model_get_value_at(data, 1, i);
		cuantia = gda_value_stringify(v);
		valor = g_strtod(cuantia, NULL);
		
		xmlTextWriterStartElement(writer, "partida");
		xmlTextWriterWriteAttribute(writer, "cuenta", cuenta);
		xmlTextWriterWriteAttribute(writer, "cuantia", cuantia);
		xmlTextWriterEndElement(writer);
		
		// Añado la linea de la partida
		gtk_list_store_append (store, &iter);
		gtk_list_store_set (store, &iter, 0, "", 1, "- XXX -", 2, "", 3, "", -1);
		path = gtk_tree_model_get_path (GTK_TREE_MODEL (store), &iter);
		text_path = gtk_tree_path_to_string(path);
		
		// Edito la cuenta
		col = gtk_tree_view_get_column (treeview, 0);
		l = gtk_tree_view_column_get_cell_renderers (col);
		diario_editar_cuenta(l->data, text_path, cuenta, treeview);
		g_list_free(l);
		
		// Edito la cuantia
		if (valor > 0) col = gtk_tree_view_get_column (treeview, 2);
		else col = gtk_tree_view_get_column (treeview, 3);
		g_snprintf(buffer, MAX_BUFFER, "%.4f", ABS(valor));
		l = gtk_tree_view_column_get_cell_renderers (col);
		diario_editar_cuantia(l->data, text_path, buffer, window);
		g_list_free(l);
		
		// Al editar correctamente una cuantia se añade una partida
		// automaticamente, eliminarla si la ultima mesa - XXX -
		// comprobar el cursor con el iter siguiente al ultimo

		gtk_tree_view_get_cursor (treeview, &path1, NULL);
		if (gtk_tree_path_compare(path, path1) != 0)
			diario_barra_remove(NULL, window);
		
		g_free(cuenta);
		g_free(cuantia);
	}
	
	xmlTextWriterEndElement(writer);
	xmlTextWriterEndDocument(writer);
    xmlFreeTextWriter(writer);	
	
	g_date_strftime(bufferfecha1, 11, "%d/%m/%y", gconta_main.ejercicio_ini);
	g_date_strftime(bufferfecha2, 11, "%d/%m/%y", gconta_main.ejercicio_fin);
	g_snprintf(buffer, MAX_BUFFER, "Asiento de cierre correspondiente al ejercicio %s - %s",
		bufferfecha1, bufferfecha2);
	textview = g_object_get_data (G_OBJECT (child), "textview1");
	textbuffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(textview));
	gtk_text_buffer_set_text(textbuffer, buffer, -1);

	gtk_widget_grab_focus(GTK_WIDGET(treeview));
}



void
exportar_pgc (GtkMenuItem *mi, gpointer window)
{
	GtkWidget *dialog;
	GtkWidget *child;
	GdaCommand *cmd;
	GdaDataModel *data;
	GdaValue *v;
	gchar *filename, *cuenta, *nombre, *padre, *descripcion;
	gchar buffer[MAX_BUFFER];
    xmlTextWriterPtr writer;
	gint i;

	g_return_if_fail(gconta_conectado());
	dialog = gtk_file_chooser_dialog_new ("Seleccione un nombre de archivo para PGC",
				      GTK_WINDOW(window),
				      GTK_FILE_CHOOSER_ACTION_SAVE,
				      GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
				      GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT,
				      NULL);
	if (gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_ACCEPT) {
		filename = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(dialog));
		if (filename == NULL) {
			gtk_widget_destroy(dialog);
			return;
		}
		if (g_file_test(filename, G_FILE_TEST_EXISTS)) {
			child = create_dialog("El fichero ya existe. ¿Desea sobre escribirlo?", FALSE);
			if (gtk_dialog_run(GTK_DIALOG(child)) != GTK_RESPONSE_OK) {
				gtk_widget_destroy(child);
				gtk_widget_destroy(dialog);
				return;
			}
		}
		
		g_snprintf(buffer, MAX_BUFFER, "SELECT P.cuenta, nombre, padre, descripcion "
				"FROM (PGC%d AS P LEFT JOIN PGC_DESCRIPCION%d AS PD ON P.cuenta = PD.cuenta)",
				gconta_main.year, gconta_main.year);
		cmd = gda_command_new(buffer, GDA_COMMAND_TYPE_SQL, GDA_COMMAND_OPTION_STOP_ON_ERRORS);
		data = gda_connection_execute_single_command(gconta_main.conn, cmd, NULL);
		gda_command_free(cmd);
		if (data == NULL) {
			dialog = create_dialog("Fallo al consultar la base de datos", TRUE);
			gconta_errores();
			gtk_dialog_run(GTK_DIALOG(dialog));
			return;
		}

		// Crear archivo
		writer = xmlNewTextWriterFilename(filename, 0);
		g_free(filename);
		if (writer == NULL) {
			child = create_dialog("No se pudo crear el archivo", TRUE);
			gtk_dialog_run(GTK_DIALOG(child));
			gtk_widget_destroy(child);
			return;
		}	
		
		xmlTextWriterStartDocument(writer, NULL, "UTF-8", NULL);
		xmlTextWriterStartDTD(writer, "pgc", NULL, NULL);
		xmlTextWriterWriteString(writer, 
			"<!ELEMENT pgc (item_pgc+)>"
			"<!ELEMENT item_pgc EMPTY>"
			"<!ATTLIST item_pgc "
			"cod CDATA #REQUIRED "
			"name CDATA #REQUIRED "
			"dad CDATA #IMPLIED "
			"descripcion CDATA #IMPLIED>");
		xmlTextWriterEndDTD(writer);
		xmlTextWriterStartElement(writer, "pgc");
		
		for(i = 0; i < gda_data_model_get_n_rows(data); i++) {
			v = (GdaValue *) gda_data_model_get_value_at(data, 0, i);
			cuenta = gda_value_stringify(v);
			v = (GdaValue *) gda_data_model_get_value_at(data, 1, i);
			nombre = gda_value_stringify(v);
			v = (GdaValue *) gda_data_model_get_value_at(data, 2, i);
			if (v == NULL || gda_value_is_null(v)) padre = NULL;
			else padre = gda_value_stringify(v);
			v = (GdaValue *) gda_data_model_get_value_at(data, 3, i);
			if (v == NULL || gda_value_is_null(v)) descripcion = NULL;
			else descripcion = gda_value_stringify(v);
			
			
			xmlTextWriterStartElement(writer, "item_pgc");
			xmlTextWriterWriteAttribute(writer, "cod", cuenta);
			xmlTextWriterWriteAttribute(writer, "name", nombre);
			if (padre != NULL) xmlTextWriterWriteAttribute(writer, "dad", padre);
			if (descripcion != NULL) xmlTextWriterWriteAttribute(writer, "descripcion", descripcion);
			xmlTextWriterEndElement(writer);
			
			g_free(cuenta);
			g_free(nombre);
			if (padre != NULL) g_free(padre);
			if (descripcion != NULL) g_free(descripcion);
		}
		
		xmlTextWriterEndElement(writer);
		xmlTextWriterEndDocument(writer);
		xmlFreeTextWriter(writer);	
	}
	gtk_widget_destroy(dialog);
}
