/*
 * xml.c
 * This file is part of gconta
 *
 * Copyright (C) 2004 Bruno Orcha Garcia
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, 
 * Boston, MA 02111-1307, USA. 
 */

#include "xml.h"
#include "pgc.h"
#include "diario.h"
#include <glib/gprintf.h>


xmlDocPtr
xml_abrir(gchar *nombre)
{
	xmlDocPtr doc;
	xmlParserCtxtPtr pc;
	gchar *file;
	
	file = gconta_uri(nombre);
	pc = xmlNewParserCtxt();
	doc = xmlCtxtReadFile(pc, file, NULL, XML_PARSE_DTDVALID);
	g_free(file);
	
	if (doc == NULL) 
		g_warning("No se pudo abrir el fichero\n");
	else if (pc->valid == 0) {
		g_warning("Fichero no válido\n");	
		xmlFreeDoc(doc);
		doc = NULL;
	} else
		xmlFreeParserCtxt(pc);
	
	return doc;	
}

void
xml_pgc_modos(GtkListStore* store)
{
	GtkTreeIter iter;
	xmlDocPtr doc;
	xmlNodePtr cur;
	xmlChar *name;
	
	doc = xml_abrir("modes.xml");
	if (doc == NULL) return;
	cur = xmlDocGetRootElement(doc);

	gtk_list_store_append(store, &iter);
	gtk_list_store_set(store, &iter, 0, "", -1);
	for(cur = cur->children; cur != NULL; cur = cur->next) 
		if (!xmlStrcmp(cur->name, (const xmlChar *)"mode")) {
			name = xmlGetProp(cur, "name");
			gtk_list_store_append(store, &iter);
			gtk_list_store_set(store, &iter, 0, name, -1);
			xmlFree(name);
		}
	
	xmlFreeDoc(doc);
}


void
xml_pgc_modos_cargar_nodos(GtkTreeStore *store, xmlNodePtr cur, GtkTreeIter *iter_parent)
{
	xmlChar *name1, *name2, *name3;
	GtkTreeIter iter;
	gint ini, fin, mas;
	
	for(cur = cur->children; cur != NULL; cur = cur->next)
		if (!xmlStrcmp(cur->name, (const xmlChar *)"node")) {
			name1 = xmlGetProp(cur, "name");
			gtk_tree_store_append(store, &iter, iter_parent);
			gtk_tree_store_set(store, &iter, 0, name1, -1);
			xml_pgc_modos_cargar_nodos(store, cur, &iter);
			xmlFree(name1);
		} else if (!xmlStrcmp(cur->name, (const xmlChar *)"list")) {
			name1 = xmlGetProp(cur, "ini");
			name2 = xmlGetProp(cur, "fin");
			name3 = xmlGetProp(cur, "mas");
			ini = (gint) g_strtod(name1, NULL);
			fin = (gint) g_strtod(name2, NULL);
			if (name3 != NULL) mas = (gint) g_strtod(name3, NULL);
			else mas = -1;
			
			pgc_llenar_nodos(store, iter_parent, ini, fin, mas);
			
			xmlFree(name1);
			xmlFree(name2);
			if (name3 != NULL) xmlFree(name3);
		}
}

void
xml_pgc_modos_cargar(GtkTreeStore *store, gchar *nombre)
{
	xmlDocPtr doc;
	xmlNodePtr cur;
	xmlChar *name;
	
	doc = xml_abrir("modes.xml");
	if (doc == NULL) return;
	cur = xmlDocGetRootElement(doc);

	for(cur = cur->children; cur != NULL; cur = cur->next) 
		if (!xmlStrcmp(cur->name, (const xmlChar *)"mode")) {
			name = xmlGetProp(cur, "name");
			if (!xmlStrcmp(name, nombre)) {
				xml_pgc_modos_cargar_nodos(store, cur, NULL);
				break;
			}
			xmlFree(name);
		}
	
	xmlFreeDoc(doc);
}

void
xml_diario_predefinidos (GtkListStore* store)
{
	GtkTreeIter iter;
	xmlDocPtr doc;
	xmlNodePtr cur;
	xmlChar *name;
	
	doc = xml_abrir("predefinidos.xml");
	if (doc == NULL) return;
	cur = xmlDocGetRootElement(doc);

	gtk_list_store_append(store, &iter);
	gtk_list_store_set(store, &iter, 0, "", -1);

	for(cur = cur->children; cur != NULL; cur = cur->next) 
		if (!xmlStrcmp(cur->name, (const xmlChar *)"predefinido")) {
			name = xmlGetProp(cur, "name");
			gtk_list_store_append(store, &iter);
			gtk_list_store_set(store, &iter, 0, name, -1);
			xmlFree(name);
		}
	
	xmlFreeDoc(doc);
}

void
xml_diario_predefinidos_llenar(GList **ts, GList **tp, gchar *nombre)
{
	xmlDocPtr doc;
	xmlNodePtr cur;
	xmlChar *name;
	
	doc = xml_abrir("predefinidos.xml");
	if (doc == NULL) return;
	cur = xmlDocGetRootElement(doc);

	for(cur = cur->children; cur != NULL; cur = cur->next) 
		if (!xmlStrcmp(cur->name, (const xmlChar *)"predefinido")) {
			name = xmlGetProp(cur, "name");
			if (!xmlStrcmp(name, nombre)) {
				xml_diario_predefinidos_llenar_nodos(ts, tp, cur);
				break;
			}
			xmlFree(name);
		}
	
	xmlFreeDoc(doc);
}


void
xml_diario_predefinidos_llenar_nodos(GList **ts, GList **tp, xmlNodePtr cur)
{
	xmlChar *name1, *name2, *name3;
	xmlNodePtr cur2;
	tsimb *t;
	tpartidas *p;
	
	for(cur = cur->children; cur != NULL; cur = cur->next)
		if (!xmlStrcmp(cur->name, (const xmlChar *)"partidas"))
			for(cur2 = cur->children; cur2 != NULL; cur2 = cur2->next) {
				if (!xmlStrcmp(cur2->name, (const xmlChar *)"partida")) {
					name1 = xmlGetProp(cur2, "cuenta");
					name2 = xmlGetProp(cur2, "cuantia");
					name3 = xmlGetProp(cur2, "tipo");
					p = g_new(tpartidas, 1);
					p->cuenta = g_strdup(name1);
					p->cuantia = g_strdup(name2);
					if (strcmp(name3, "debe") == 0) p->debe = TRUE;
					else p->debe = FALSE;
					*tp = g_list_append(*tp, p);
					xmlFree(name1);
					xmlFree(name2);
					xmlFree(name3);
				}
		} else if (!xmlStrcmp(cur->name, (const xmlChar *)"variables")) 
			for(cur2 = cur->children; cur2 != NULL; cur2 = cur2->next) {
				if (!xmlStrcmp(cur2->name, (const xmlChar *)"variable")) {				
					name1 = xmlGetProp(cur2, "name");
					name2 = xmlGetProp(cur2, "question");
					t = g_new(tsimb, 1);
					t->val = 0;
					t->var = g_strdup(name1);
					t->question = g_strdup(name2);					
					*ts = g_list_append(*ts, t);
					xmlFree(name1);
					xmlFree(name2);
				}
			}
}


void
xml_consultas_cuenta(gchar *cuenta, gchar **nombre, gchar **descripcion)
{
	xmlDocPtr doc;
	xmlNodePtr cur;
	xmlChar *codigo;

	*nombre = NULL;
	*descripcion = NULL;
	
	doc = xml_abrir("pgcdescripcion.xml");
	if (doc == NULL) return;
	cur = xmlDocGetRootElement(doc);

	for(cur = cur->children; cur != NULL; cur = cur->next) 
		if (!xmlStrcmp(cur->name, (const xmlChar *)"cuenta")) {
			codigo = xmlGetProp(cur, "codigo");
			if (!xmlStrcmp(codigo, cuenta)) {
				*nombre = xmlGetProp(cur, "name");
				*descripcion = xmlNodeGetContent(cur);
				xmlFree(codigo);
				break;
			}
			xmlFree(codigo);
		}
	
	xmlFreeDoc(doc);
}

gchar *
xml_empresa_nombre(const gchar *l) // Directorio
{
	xmlParserCtxtPtr pc;
	xmlDocPtr doc;
	xmlNodePtr cur;
	gchar *path, *file, *ejer = NULL;
	
	file = g_build_filename(g_get_home_dir(), G_DIR_SEPARATOR_S, ".gconta",
			G_DIR_SEPARATOR_S, l, G_DIR_SEPARATOR_S, "empresa.xml", NULL);
	if (g_file_test(file, G_FILE_TEST_EXISTS) == FALSE) {
		g_free(file);
		return NULL;
	}
		
	pc = xmlNewParserCtxt();
	doc = xmlCtxtReadFile(pc, file, NULL, XML_PARSE_DTDVALID);
	g_free(file);
	path = NULL;
	if (doc != NULL && pc->valid != 0) {
		cur = xmlDocGetRootElement(doc);
		for(cur = cur->children; cur != NULL; cur = cur->next) 
			if (!xmlStrcmp(cur->name, (const xmlChar *)"nombre")) 
				path = xmlNodeGetContent(cur);
			else if (!xmlStrcmp(cur->name, (const xmlChar *)"ejercicio")) 
				ejer = xmlNodeGetContent(cur);
		xmlFreeDoc(doc);
	}
	xmlFreeParserCtxt(pc);
	
	file = g_strdup_printf("%s (%s)", path, ejer);
	
	return file;
}

gint
xml_cargar_gconta_main(gchar *directorio)
{
	xmlParserCtxtPtr pc;
	xmlDocPtr doc;
	xmlNodePtr cur;
	gchar *file, *fecha;
	

	file = g_build_filename(g_get_home_dir(), G_DIR_SEPARATOR_S, ".gconta",
		G_DIR_SEPARATOR_S, directorio, G_DIR_SEPARATOR_S, "empresa.xml", NULL);
	if (file == NULL) {
		g_warning("No se encontró empresa.xml");
		return -1;
	}

	gconta_main.error_dialog = NULL;
	
	pc = xmlNewParserCtxt();
	doc = xmlCtxtReadFile(pc, file, NULL, XML_PARSE_DTDVALID);
	g_free(file);
	
	if (doc != NULL && pc->valid != 0) {
		xmlFreeParserCtxt(pc);
		cur = xmlDocGetRootElement(doc);
		for(cur = cur->children; cur != NULL; cur = cur->next) 
			if (!xmlStrcmp(cur->name, (const xmlChar *)"nombre")) 
				gconta_main.nombre_empresa = xmlNodeGetContent(cur);
			else if (!xmlStrcmp(cur->name, (const xmlChar *)"alias")) 
				gconta_main.alias = xmlNodeGetContent(cur);			
			else if (!xmlStrcmp(cur->name, (const xmlChar *)"descripcion")) 
				gconta_main.descripcion = xmlNodeGetContent(cur);
			else if (!xmlStrcmp(cur->name, (const xmlChar *)"gdasource")) 
				gconta_main.gda_source = xmlNodeGetContent(cur);			
			else if (!xmlStrcmp(cur->name, (const xmlChar *)"ejercicio")) {
				file = xmlNodeGetContent(cur);
				gconta_main.year = (gint) g_ascii_strtod(file, NULL);
				g_free(file);
				fecha = g_strdup_printf("%d-1-1", gconta_main.year);
				gconta_main.ejercicio_ini = g_date_new();
				g_date_set_parse(gconta_main.ejercicio_ini, fecha);
				g_free(fecha);
				fecha = g_strdup_printf("%d-12-31", gconta_main.year);
				gconta_main.ejercicio_fin = g_date_new();
				g_date_set_parse(gconta_main.ejercicio_fin, fecha);
				g_free(fecha);
			}
		xmlFreeDoc(doc);
	} else {
		g_warning("Fallo al abrir empresa.xml");
		return -1;
	}
	
	return 0;
}


void
xml_cargar_pgc(gint year)
{
	GtkWidget *dialog;
	xmlDocPtr doc;
	xmlNodePtr cur;
	xmlChar *cod;
	xmlChar *name;
	xmlChar *dad;
	xmlChar *descripcion;
	gchar buffer[MAX_BUFFER];
	gchar *padre;
	GdaTransaction *trans;
	
	gconta_main.year = year;
	doc = xml_abrir("pgc.xml");
		
	if (doc == NULL) {
		g_warning("No se pudo abrir pgc.xml");
		return;
	}
	
	if (!gda_connection_supports(gconta_main.conn,
		GDA_CONNECTION_FEATURE_TRANSACTIONS))
	{
		dialog = create_dialog ("El SGBD no soporta transacciones!!", TRUE);
		gtk_dialog_run (GTK_DIALOG (dialog));
		gtk_widget_destroy (dialog);
		return;
	}
	
	/* comienza transaccion */
	trans = gda_transaction_new ("insertar_pgc");
	gda_transaction_set_isolation_level (trans,
						 GDA_TRANSACTION_ISOLATION_SERIALIZABLE);
	if (!gda_connection_begin_transaction (gconta_main.conn, trans)) {
		gconta_errores ();
		dialog = create_dialog ("No se pudo comenzar la transacción", TRUE);
		gtk_dialog_run (GTK_DIALOG (dialog));
		gtk_widget_destroy (dialog);
		return;
	}
	
	cur = xmlDocGetRootElement(doc);
	for(cur = cur->children; cur != NULL; cur = cur->next)
		if (!xmlStrcmp(cur->name, (const xmlChar *)"item_pgc")) {
			cod = xmlGetProp(cur, "cod");
			name = xmlGetProp(cur, "name");
			dad = xmlGetProp(cur, "dad");
			descripcion = xmlGetProp(cur, "descripcion");
			if (dad == NULL) padre = "NULL";
			else padre = dad;

			/* Insertar en PGC */
			g_snprintf(buffer, MAX_BUFFER, "INSERT INTO PGC%d (cuenta, nombre, padre) VALUES (%s, '%s', %s);",
					year, cod, name, padre);
			GCONTA_nonquery(buffer, "Ocurrió un error al guardar la cuenta", trans);
			
			/* Si hay algo en descripcion insertar en PGC_DESCRIPCION */
			if (descripcion != NULL) {
				g_snprintf (buffer, MAX_BUFFER,
					   "INSERT INTO PGC_DESCRIPCION%d (cuenta, descripcion) VALUES (%s, '%s')",
					   gconta_main.year, cod, descripcion);
				xmlFree(descripcion);
				GCONTA_nonquery (buffer, "Ocurrió un error al guardar la descripción", trans);
			}
	
			xmlFree(cod);
			xmlFree(name);
			xmlFree(dad);
		}
	xmlFreeDoc(doc);
		
	gda_connection_commit_transaction (gconta_main.conn, trans);
	g_object_unref (trans);

}


void
xml_informes (GtkListStore* store)
{
	GtkTreeIter iter;
	xmlDocPtr doc;
	xmlNodePtr cur;
	xmlChar *name;
	
	doc = xml_abrir("informes.xml");
	if (doc == NULL) return;
	cur = xmlDocGetRootElement(doc);

	gtk_list_store_append(store, &iter);
	gtk_list_store_set(store, &iter, 0, "", -1);

	for(cur = cur->children; cur != NULL; cur = cur->next) 
		if (!xmlStrcmp(cur->name, (const xmlChar *)"informe")) {
			name = xmlGetProp(cur, "name");
			gtk_list_store_append(store, &iter);
			gtk_list_store_set(store, &iter, 0, name, -1);
			xmlFree(name);
		}
	
	xmlFreeDoc(doc);
}


void
xml_informes_llenar(GList **lista, gchar *nombre)
{
	xmlDocPtr doc;
	xmlNodePtr cur;
	xmlChar *name;
	
	doc = xml_abrir("informes.xml");
	if (doc == NULL) return;
	cur = xmlDocGetRootElement(doc);

	for(cur = cur->children; cur != NULL; cur = cur->next) 
		if (!xmlStrcmp(cur->name, (const xmlChar *)"informe")) {
			name = xmlGetProp(cur, "name");
			if (!xmlStrcmp(name, nombre))
				xml_informes_llenar_nodos(lista, cur);
			xmlFree(name);
		}
	
	xmlFreeDoc(doc);
}

gboolean
xml_informes_comprobar_ok_hijos(GList *hijos, gdouble *valor)
{
	gboolean ok;
	gdouble val;
	tinforme *tip;
	
	ok = TRUE;
	val = 0.0;
	
	for(hijos = g_list_first(hijos); ok && hijos != NULL; hijos = g_list_next(hijos))	{
		tip = (tinforme *) hijos->data;
		if (tip->ok) val += tip->valor;
		else ok = FALSE;
	}

	*valor = val;
	return ok;
}

void
xml_informes_llenar_nodos(GList **lista, xmlNodePtr cur)
{
	xmlNodePtr cur2;
	gchar *cuenta, *ant, *aux;
	tinforme *tip;
	
	for(cur = cur->children; cur != NULL; cur = cur->next) 
		if (!xmlStrcmp(cur->name, (const xmlChar *)"node")) {
			tip = (tinforme *) g_new(tinforme, 1);
			tip->var = xmlGetProp(cur, "var");
			tip->texto = xmlGetProp(cur, "name");
			tip->valor = 0;
			tip->valor_ant = 0;
			tip->hijos = NULL;
			tip->cuentas = NULL;
			tip->bold = FALSE; // sup not is sum
			tip->ok = FALSE;
			*lista = g_list_append(*lista, tip);
			// 3 opciones
			for(cur2 = cur->children; cur2 != NULL; cur2 = cur2->next) {
				if (!xmlStrcmp(cur2->name, (const xmlChar *)"node")) {
					xml_informes_llenar_nodos(&(tip->hijos), cur2->parent);
					tip->ok = xml_informes_comprobar_ok_hijos(tip->hijos, &(tip->valor));
					break;
				} else if (!xmlStrcmp(cur2->name, (const xmlChar *)"suma")) {
					// Es una suma
					tip->cuentas = xmlNodeGetContent(cur2);
					tip->bold = TRUE;
				} else if (!xmlStrcmp(cur2->name, (const xmlChar *)"cuenta")) {
					// Hay una lista de cuentas
					aux = NULL;
					ant = NULL;
					for(; cur2 != NULL; cur2 = cur2->next) {
						if (!xmlStrcmp(cur2->name, (const xmlChar *)"cuenta")) {
							cuenta = xmlNodeGetContent(cur2);
							if (ant == NULL) ant = g_strdup(cuenta);
							else {
								aux = g_strdup_printf("%s, %s", ant, cuenta);
								g_free(ant);
								ant = aux;
							}
							tip->valor += gconta_saldo(g_ascii_strtod(cuenta, NULL), TRUE, NULL);
							g_free(cuenta);
						}
					}
					tip->cuentas = ant;
					tip->ok = TRUE;
					aux = xmlGetProp(cur, "tipo");
					if (aux != NULL && !xmlStrcmp(aux, (const xmlChar *)"debe")) {
						tip->valor = -tip->valor;
						//tip->valor_ant = -tip->valor_ant;
					}
					break;
				}
			}
		} else if (!xmlStrcmp(cur->name, (const xmlChar *)"salto")) {
			tip = (tinforme *) g_new(tinforme, 1);
			tip->var = NULL;
			tip->texto = xmlNodeGetContent(cur);
			tip->valor = 0;
			tip->valor_ant = 0;
			tip->hijos = NULL;
			tip->cuentas = NULL;
			tip->bold = FALSE;
			tip->ok = TRUE;
			*lista = g_list_append(*lista, tip);
		}
}
